@@ -82,6 +82,10 @@ class VersionNegotiationFilter(wsgi.Middleware):
         allowed_versions['v2.6'] = 2
         allowed_versions['v2.7'] = 2
         allowed_versions['v2.9'] = 2
+        allowed_versions['v2.13'] = 2
+        if CONF.image_cache_dir:
+            allowed_versions['v2.14'] = 2
+        allowed_versions['v2.15'] = 2
         if CONF.enabled_backends:
             allowed_versions['v2.8'] = 2
             allowed_versions['v2.10'] = 2
