@@ -22,7 +22,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-import six
 import webob.exc
 
 import glance.api.policy
@@ -189,7 +188,7 @@ class ImageDataController(object):
                          "The image may have been deleted during the "
                          "upload, cleaning up the chunks uploaded.") %
                        image_id)
-                LOG.warn(msg)
+                LOG.warning(msg)
                 # NOTE(sridevi): Cleaning up the uploaded chunks.
                 try:
                     image.delete()
@@ -573,7 +572,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             response.headers['Content-MD5'] = image.checksum
         # NOTE(markwash): "response.app_iter = ..." also erroneously resets the
         # content-length
-        response.headers['Content-Length'] = six.text_type(chunk_size)
+        response.headers['Content-Length'] = str(chunk_size)
 
     def upload(self, response, result):
         response.status_int = 204
