@@ -1,4 +1,3 @@
-# Copyright 2012 OpenStack Foundation.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,8 +14,10 @@
 
 import datetime
 import hashlib
+import http.client as http
 import os
 import re
+import urllib.parse as urlparse
 
 from castellan.common import exception as castellan_exception
 from castellan import key_manager
@@ -27,24 +28,24 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils as json
 from oslo_utils import encodeutils
 from oslo_utils import timeutils as oslo_timeutils
-import six
-from six.moves import http_client as http
-import six.moves.urllib.parse as urlparse
+import requests
 import webob.exc
 
-from glance.api import authorization
 from glance.api import common
 from glance.api import policy
+from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import location_strategy
 from glance.common import store_utils
 from glance.common import timeutils
 from glance.common import utils
 from glance.common import wsgi
+from glance import context as glance_context
 import glance.db
 import glance.gateway
-from glance.i18n import _, _LI, _LW
+from glance.i18n import _, _LE, _LI, _LW
 import glance.notifier
+from glance.quota import keystone as ks_quota
 import glance.schema
 
 LOG = logging.getLogger(__name__)
@@ -57,6 +58,24 @@ CONF.import_opt('show_multiple_locations', 'glance.common.config')
 CONF.import_opt('hashing_algorithm', 'glance.common.config')
 
 
+def proxy_response_error(orig_code, orig_explanation):
+    """Construct a webob.exc.HTTPError exception on the fly.
+
+    The webob.exc.HTTPError classes are statically defined, intended
+    to be straight subclasses of HTTPError, specifically with *class*
+    level definitions of things we need to be dynamic. This method
+    returns an exception class instance with those values set
+    programmatically so we can raise it to mimic the response we
+    got from a remote.
+    """
+
+    class ProxiedResponse(webob.exc.HTTPError):
+        code = orig_code
+        title = orig_explanation
+
+    return ProxiedResponse()
+
+
 class ImagesController(object):
     def __init__(self, db_api=None, policy_enforcer=None, notifier=None,
                  store_api=None):
@@ -71,9 +90,18 @@ class ImagesController(object):
 
     @utils.mutating
     def create(self, req, image, extra_properties, tags):
-        image_factory = self.gateway.get_image_factory(req.context)
-        image_repo = self.gateway.get_repo(req.context)
+        image_factory = self.gateway.get_image_factory(
+            req.context, authorization_layer=False)
+        image_repo = self.gateway.get_repo(req.context,
+                                           authorization_layer=False)
         try:
+            if 'owner' not in image:
+                image['owner'] = req.context.project_id
+
+            api_policy.ImageAPIPolicy(req.context, image,
+                                      self.policy).add_image()
+
+            ks_quota.enforce_image_count_total(req.context, req.context.owner)
             image = image_factory.new_image(extra_properties=extra_properties,
                                             tags=tags, **image)
             image_repo.add(image)
@@ -87,7 +115,7 @@ class ImagesController(object):
             LOG.debug("User not permitted to create image")
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.LimitExceeded as e:
-            LOG.warn(encodeutils.exception_to_unicode(e))
+            LOG.warning(encodeutils.exception_to_unicode(e))
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=e.msg, request=req, content_type='text/plain')
         except exception.Duplicate as e:
@@ -138,8 +166,10 @@ class ImagesController(object):
 
     def _enforce_import_lock(self, req, image):
         admin_context = req.context.elevated()
-        admin_image_repo = self.gateway.get_repo(admin_context)
-        admin_task_repo = self.gateway.get_task_repo(admin_context)
+        admin_image_repo = self.gateway.get_repo(
+            admin_context, authorization_layer=False)
+        admin_task_repo = self.gateway.get_task_repo(
+            admin_context, authorization_layer=False)
         other_task = image.extra_properties['os_glance_import_task']
 
         expiry = datetime.timedelta(minutes=60)
@@ -211,16 +241,95 @@ class ImagesController(object):
                       {'image': image.image_id, 'task': task.task_id,
                        'keys': ','.join(changed)})
 
+    def _proxy_request_to_stage_host(self, image, req, body=None):
+        """Proxy a request to a staging host.
+
+        When an image was staged on another worker, that worker may record its
+        worker_self_reference_url on the image, indicating that other workers
+        should proxy requests to it while the image is staged. This method
+        replays our current request against the remote host, returns the
+        result, and performs any response error translation required.
+
+        The remote request-id is used to replace the one on req.context so that
+        a client sees the proper id used for the actual action.
+
+        :param image: The Image from the repo
+        :param req: The webob.Request from the current request
+        :param body: The request body or None
+        :returns: The result from the remote host
+        :raises: webob.exc.HTTPClientError matching the remote's error, or
+                 webob.exc.HTTPServerError if we were unable to contact the
+                 remote host.
+        """
+
+        stage_host = image.extra_properties['os_glance_stage_host']
+        LOG.info(_LI('Proxying %s request to host %s '
+                     'which has image staged'),
+                 req.method, stage_host)
+        client = glance_context.get_ksa_client(req.context)
+        url = '%s%s' % (stage_host, req.path)
+        req_id_hdr = 'x-openstack-request-id'
+        request_method = getattr(client, req.method.lower())
+        try:
+            r = request_method(url, json=body, timeout=60)
+        except (requests.exceptions.ConnectionError,
+                requests.exceptions.ConnectTimeout) as e:
+            LOG.error(_LE('Failed to proxy to %r: %s'), url, e)
+            raise webob.exc.HTTPGatewayTimeout('Stage host is unavailable')
+        except requests.exceptions.RequestException as e:
+            LOG.error(_LE('Failed to proxy to %r: %s'), url, e)
+            raise webob.exc.HTTPBadGateway('Stage host is unavailable')
+        req_id_hdr = 'x-openstack-request-id'
+        if req_id_hdr in r.headers:
+            LOG.debug('Replying with remote request id %s' % (
+                r.headers[req_id_hdr]))
+            req.context.request_id = r.headers[req_id_hdr]
+        if r.status_code // 100 != 2:
+            raise proxy_response_error(r.status_code, r.reason)
+        return image.image_id
+
+    @property
+    def self_url(self):
+        """Return the URL we expect to point to us.
+
+        If this is set to a per-worker URL in worker_self_reference_url,
+        that takes precedence. Otherwise we fall back to public_endpoint.
+        """
+        return CONF.worker_self_reference_url or CONF.public_endpoint
+
+    def is_proxyable(self, image):
+        """Decide if an action is proxyable to a stage host.
+
+        If the image has a staging host recorded with a URL that does not match
+        ours, then we can proxy our request to that host.
+
+        :param image: The Image from the repo
+        :returns: bool indicating proxyable status
+        """
+        return (
+            'os_glance_stage_host' in image.extra_properties and
+            image.extra_properties['os_glance_stage_host'] != self.self_url)
+
     @utils.mutating
     def import_image(self, req, image_id, body):
-        image_repo = self.gateway.get_repo(req.context)
-        task_factory = self.gateway.get_task_factory(req.context)
-        task_repo = self.gateway.get_task_repo(req.context)
+        ctxt = req.context
+        image_repo = self.gateway.get_repo(ctxt,
+                                           authorization_layer=False)
+        task_factory = self.gateway.get_task_factory(
+            ctxt, authorization_layer=False)
+        task_repo = self.gateway.get_task_repo(
+            ctxt, authorization_layer=False)
         import_method = body.get('method').get('name')
         uri = body.get('method').get('uri')
         all_stores_must_succeed = body.get('all_stores_must_succeed', True)
         stole_lock_from_task = None
 
+        try:
+            ks_quota.enforce_image_size_total(req.context, req.context.owner)
+        except exception.LimitExceeded as e:
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=str(e),
+                                                      request=req)
+
         try:
             image = image_repo.get(image_id)
             if image.status == 'active' and import_method != "copy-image":
@@ -249,12 +358,15 @@ class ImagesController(object):
             # NOTE(danms): For copy-image only, we check policy to decide
             # if the user should be able to do this. Otherwise, we forbid
             # the import if the user is not the owner.
+
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                enforcer=self.policy)
             if import_method == 'copy-image':
-                self.policy.enforce(req.context, 'copy_image',
-                                    dict(policy.ImageTarget(image)))
-            elif not authorization.is_image_mutable(req.context, image):
-                raise webob.exc.HTTPForbidden(
-                    explanation=_("Operation not permitted"))
+                api_pol.copy_image()
+            else:
+                # NOTE(abhishekk): We need to perform ownership check on image
+                # so that non-admin or non-owner can not import data to image
+                api_pol.modify_image()
 
             if 'os_glance_import_task' in image.extra_properties:
                 # NOTE(danms): This will raise exception.Conflict if the
@@ -266,7 +378,7 @@ class ImagesController(object):
                 try:
                     stores = utils.get_stores_from_request(req, body)
                 except glance_store.UnknownScheme as exc:
-                    LOG.warn(exc.msg)
+                    LOG.warning(exc.msg)
                     raise exception.Conflict(exc.msg)
 
             # NOTE(abhishekk): If all_stores is specified and import_method is
@@ -308,6 +420,12 @@ class ImagesController(object):
                      "enabled_backends %s") % uri)
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
+        if self.is_proxyable(image) and import_method == 'glance-direct':
+            # NOTE(danms): Image is staged on another worker; proxy the
+            # import request to that worker with the user's token, as if
+            # they had called it themselves.
+            return self._proxy_request_to_stage_host(image, req, body)
+
         task_input = {'image_id': image_id,
                       'import_req': body,
                       'backend': stores}
@@ -316,12 +434,12 @@ class ImagesController(object):
             # If this is a copy-image import and we passed the policy check,
             # grab an admin context for the task so it can manipulate metadata
             # as admin.
-            admin_context = req.context.elevated()
+            admin_context = ctxt.elevated()
         else:
             admin_context = None
 
         executor_factory = self.gateway.get_task_executor_factory(
-            req.context, admin_context=admin_context)
+            ctxt, admin_context=admin_context, authorization_layer=False)
 
         if (import_method == 'web-download' and
                 not utils.validate_import_uri(uri)):
@@ -331,8 +449,11 @@ class ImagesController(object):
 
         try:
             import_task = task_factory.new_task(task_type='api_image_import',
-                                                owner=req.context.owner,
-                                                task_input=task_input)
+                                                owner=ctxt.owner,
+                                                task_input=task_input,
+                                                image_id=image_id,
+                                                user_id=ctxt.user_id,
+                                                request_id=ctxt.request_id)
 
             # NOTE(danms): Try to grab the lock for this task
             try:
@@ -353,7 +474,7 @@ class ImagesController(object):
                                                   stole_lock_from_task)
 
             task_repo.add(import_task)
-            task_executor = executor_factory.new_task_executor(req.context)
+            task_executor = executor_factory.new_task_executor(ctxt)
             pool = common.get_thread_pool("tasks_pool")
             pool.spawn(import_task.run, task_executor)
         except exception.Forbidden as e:
@@ -363,6 +484,9 @@ class ImagesController(object):
             raise webob.exc.HTTPConflict(explanation=e.msg)
         except exception.InvalidImageStatusTransition as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except exception.LimitExceeded as e:
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=str(e),
+                                                      request=req)
         except ValueError as e:
             LOG.debug("Cannot import data for image %(id)s: %(e)s",
                       {'id': image_id,
@@ -404,14 +528,31 @@ class ImagesController(object):
             limit = CONF.limit_param_default
         limit = min(CONF.api_limit_max, limit)
 
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(req.context,
+                                           authorization_layer=False)
         try:
+            # NOTE(danms): This is just a "do you have permission to
+            # list images" check. Each image is checked against
+            # get_image below.
+            target = {'project_id': req.context.project_id}
+            self.policy.enforce(req.context, 'get_images', target)
+
             images = image_repo.list(marker=marker, limit=limit,
                                      sort_key=sort_key,
                                      sort_dir=sort_dir,
                                      filters=filters,
                                      member_status=member_status)
-            if len(images) != 0 and len(images) == limit:
+            db_image_count = len(images)
+            images = [image for image in images
+                      if api_policy.ImageAPIPolicy(req.context, image,
+                                                   self.policy
+                                                   ).check('get_image')]
+
+            # NOTE(danms): we need to include the next marker if the DB
+            # paginated. Since we filter images based on policy, we can
+            # not determine if pagination happened from the final list,
+            # so use the original count.
+            if len(images) != 0 and db_image_count == limit:
                 result['next_marker'] = images[-1].image_id
         except (exception.NotFound, exception.InvalidSortKey,
                 exception.InvalidFilterRangeValue,
@@ -427,27 +568,49 @@ class ImagesController(object):
         return result
 
     def show(self, req, image_id):
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(req.context,
+                                           authorization_layer=False)
         try:
-            return image_repo.get(image_id)
-        except exception.Forbidden as e:
-            LOG.debug("User not permitted to show image '%s'", image_id)
-            raise webob.exc.HTTPForbidden(explanation=e.msg)
+            image = image_repo.get(image_id)
+            api_policy.ImageAPIPolicy(req.context, image,
+                                      self.policy).get_image()
+            return image
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.NotAuthenticated as e:
             raise webob.exc.HTTPUnauthorized(explanation=e.msg)
 
+    def get_task_info(self, req, image_id):
+        image_repo = self.gateway.get_repo(
+            req.context, authorization_layer=False)
+
+        try:
+            # NOTE (abhishekk): Just to check image is valid
+            image = image_repo.get(image_id)
+            # Check you are authorized to fetch image details
+            api_policy.ImageAPIPolicy(req.context, image,
+                                      self.policy).get_image()
+        except (exception.NotFound, exception.Forbidden):
+            raise webob.exc.HTTPNotFound()
+
+        tasks = self.db_api.tasks_get_by_image(req.context,
+                                               image.image_id)
+
+        return {"tasks": tasks}
+
     @utils.mutating
     def update(self, req, image_id, changes):
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(req.context,
+                                           authorization_layer=False)
         try:
             image = image_repo.get(image_id)
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                self.policy)
 
             for change in changes:
                 change_method_name = '_do_%s' % change['op']
                 change_method = getattr(self, change_method_name)
-                change_method(req, image, change)
+                change_method(req, image, api_pol, change)
 
             if changes:
                 image_repo.save(image)
@@ -461,7 +624,7 @@ class ImagesController(object):
         except exception.StorageQuotaFull as e:
             msg = (_("Denying attempt to upload image because it exceeds the"
                      " quota: %s") % encodeutils.exception_to_unicode(e))
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=msg, request=req, content_type='text/plain')
         except exception.LimitExceeded as e:
@@ -473,7 +636,7 @@ class ImagesController(object):
 
         return image
 
-    def _do_replace(self, req, image, change):
+    def _do_replace(self, req, image, api_pol, change):
         path = change['path']
         path_root = path[0]
         value = change['value']
@@ -481,11 +644,13 @@ class ImagesController(object):
             msg = _("Cannot set locations to empty list.")
             raise webob.exc.HTTPForbidden(msg)
         elif path_root == 'locations' and value:
+            api_pol.update_locations()
             self._do_replace_locations(image, value)
         elif path_root == 'owner' and req.context.is_admin == False:
             msg = _("Owner can't be updated by non admin.")
             raise webob.exc.HTTPForbidden(msg)
         else:
+            api_pol.update_property(path_root, value)
             if hasattr(image, path_root):
                 setattr(image, path_root, value)
             elif path_root in image.extra_properties:
@@ -494,14 +659,16 @@ class ImagesController(object):
                 msg = _("Property %s does not exist.")
                 raise webob.exc.HTTPConflict(msg % path_root)
 
-    def _do_add(self, req, image, change):
+    def _do_add(self, req, image, api_pol, change):
         path = change['path']
         path_root = path[0]
         value = change['value']
         json_schema_version = change.get('json_schema_version', 10)
         if path_root == 'locations':
+            api_pol.update_locations()
             self._do_add_locations(image, path[1], value)
         else:
+            api_pol.update_property(path_root, value)
             if ((hasattr(image, path_root) or
                     path_root in image.extra_properties)
                     and json_schema_version == 4):
@@ -512,15 +679,17 @@ class ImagesController(object):
             else:
                 image.extra_properties[path_root] = value
 
-    def _do_remove(self, req, image, change):
+    def _do_remove(self, req, image, api_pol, change):
         path = change['path']
         path_root = path[0]
         if path_root == 'locations':
+            api_pol.delete_locations()
             try:
                 self._do_remove_locations(image, path[1])
             except exception.Forbidden as e:
                 raise webob.exc.HTTPForbidden(e.msg)
         else:
+            api_pol.update_property(path_root)
             if hasattr(image, path_root):
                 msg = _("Property %s may not be removed.")
                 raise webob.exc.HTTPForbidden(msg % path_root)
@@ -547,14 +716,14 @@ class ImagesController(object):
         except castellan_exception.Forbidden:
             msg = ('Not allowed to delete encryption key %s' %
                    cinder_encryption_key_id)
-            LOG.warn(msg)
+            LOG.warning(msg)
         except (castellan_exception.ManagedObjectNotFoundError, KeyError):
             msg = 'Could not find encryption key %s' % cinder_encryption_key_id
-            LOG.warn(msg)
+            LOG.warning(msg)
         except castellan_exception.KeyManagerError:
             msg = ('Failed to delete cinder encryption key %s' %
                    cinder_encryption_key_id)
-            LOG.warn(msg)
+            LOG.warning(msg)
 
     @utils.mutating
     def delete_from_store(self, req, store_id, image_id):
@@ -565,7 +734,8 @@ class ImagesController(object):
                    store_id)
             raise webob.exc.HTTPConflict(explanation=msg)
 
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(
+            req.context, authorization_layer=False)
         try:
             image = image_repo.get(image_id)
         except exception.NotAuthenticated as e:
@@ -575,6 +745,18 @@ class ImagesController(object):
                    {'image_id': image_id})
             raise webob.exc.HTTPNotFound(explanation=msg)
 
+        # NOTE(abhishekk): Delete from store internally checks for
+        # get_image_location and delete_image_location policies using
+        # ImageLocationProxy object, so this is the right place to
+        # check those policies
+        api_pol = api_policy.ImageAPIPolicy(req.context, image, self.policy)
+        api_pol.get_image_location()
+        # This policy will check for legacy image ownership as well
+        try:
+            api_pol.delete_locations()
+        except exception.Forbidden as e:
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+
         if image.status != 'active':
             msg = _("It's not allowed to remove image data from store if "
                     "image status is not 'active'")
@@ -618,11 +800,70 @@ class ImagesController(object):
 
         image_repo.save(image)
 
+    def _delete_image_on_remote(self, image, req):
+        """Proxy an image delete to a staging host.
+
+        When an image is staged and then deleted, the staging host still
+        has local residue that needs to be cleaned up. If the request to
+        delete arrived here, but we are not the stage host, we need to
+        proxy it to the appropriate host.
+
+        If the delete succeeds, we return None (per DELETE semantics),
+        indicating to the caller that it was handled.
+
+        If the delete fails on the remote end, we allow the
+        HTTPClientError to bubble to our caller, which will return the
+        error to the client.
+
+        If we fail to contact the remote server, we catch the
+        HTTPServerError raised by our proxy method, verify that the
+        image still exists, and return it. That indicates to the
+        caller that it should proceed with the regular delete logic,
+        which will satisfy the client's request, but leave the residue
+        on the stage host (which is unavoidable).
+
+        :param image: The Image from the repo
+        :param req: The webob.Request for this call
+        :returns: None if successful, or a refreshed image if the proxy failed.
+        :raises: webob.exc.HTTPClientError if so raised by the remote server.
+        """
+        try:
+            self._proxy_request_to_stage_host(image, req)
+        except webob.exc.HTTPServerError:
+            # This means we would have raised a 50x error, indicating
+            # we did not succeed with the request to the remote host.
+            # In this case, refresh the image from the repo, and if it
+            # is not deleted, allow the regular delete process to
+            # continue on the local worker to match the user's
+            # expectations. If the image is already deleted, the caller
+            # will catch this NotFound like normal.
+            return self.gateway.get_repo(
+                req.context,
+                authorization_layer=False).get(image.image_id)
+
     @utils.mutating
     def delete(self, req, image_id):
-        image_repo = self.gateway.get_repo(req.context)
+        image_repo = self.gateway.get_repo(req.context,
+                                           authorization_layer=False)
         try:
             image = image_repo.get(image_id)
+
+            # NOTE(abhishekk): This is the right place to check whether user
+            # have permission to delete the image and remove the policy check
+            # later from the policy layer.
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                self.policy)
+            api_pol.delete_image()
+
+            if self.is_proxyable(image):
+                # NOTE(danms): Image is staged on another worker; proxy the
+                # delete request to that worker with the user's token, as if
+                # they had called it themselves.
+                image = self._delete_image_on_remote(image, req)
+                if image is None:
+                    # Delete was proxied, so we are done here.
+                    return
+
             # NOTE(abhishekk): Delete the data from staging area
             if CONF.enabled_backends:
                 separator, staging_dir = store_utils.get_dir_separator()
@@ -668,14 +909,14 @@ class ImagesController(object):
         except (glance_store.NotFound, exception.NotFound):
             msg = (_("Failed to find image %(image_id)s to delete") %
                    {'image_id': image_id})
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise webob.exc.HTTPNotFound(explanation=msg)
         except glance_store.exceptions.InUseByStore as e:
             msg = (_("Image %(id)s could not be deleted "
                      "because it is in use: %(exc)s") %
                    {"id": image_id,
                     "exc": e.msg})
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise webob.exc.HTTPConflict(explanation=msg)
         except glance_store.exceptions.HasSnapshot as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
@@ -872,8 +1113,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
                             'size', 'virtual_size', 'direct_url', 'self',
                             'file', 'schema', 'id', 'os_hash_algo',
                             'os_hash_value')
-    _reserved_properties = ('location', 'deleted', 'deleted_at',
-                            'os_glance_import_task')
+    _reserved_properties = ('location', 'deleted', 'deleted_at')
+    _reserved_namespaces = (common.GLANCE_RESERVED_NS,)
     _base_properties = ('checksum', 'created_at', 'container_format',
                         'disk_format', 'id', 'min_disk', 'min_ram', 'name',
                         'size', 'virtual_size', 'status', 'tags', 'owner',
@@ -906,8 +1147,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         for key in cls._disallowed_properties:
             if key in image:
                 msg = _("Attribute '%s' is read-only.") % key
-                raise webob.exc.HTTPForbidden(
-                    explanation=six.text_type(msg))
+                raise webob.exc.HTTPForbidden(explanation=msg)
 
     def create(self, request):
         body = self._get_request_body(request)
@@ -939,6 +1179,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
                          "characters."))
                 raise webob.exc.HTTPBadRequest(explanation=msg)
 
+            if key in self._reserved_properties:
+                msg = _("Attribute '%s' is reserved.") % key
+                raise webob.exc.HTTPForbidden(msg)
+            if any(key.startswith(ns) for ns in self._reserved_namespaces):
+                msg = _("Attribute '%s' is reserved.") % key
+                raise webob.exc.HTTPForbidden(msg)
+
         return dict(image=image, extra_properties=properties, tags=tags)
 
     def _get_change_operation_d10(self, raw_change):
@@ -1029,10 +1276,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         path_root = change['path'][0]
         if path_root in self._readonly_properties:
             msg = _("Attribute '%s' is read-only.") % path_root
-            raise webob.exc.HTTPForbidden(explanation=six.text_type(msg))
+            raise webob.exc.HTTPForbidden(explanation=msg)
         if path_root in self._reserved_properties:
             msg = _("Attribute '%s' is reserved.") % path_root
-            raise webob.exc.HTTPForbidden(explanation=six.text_type(msg))
+            raise webob.exc.HTTPForbidden(explanation=msg)
+        if any(path_root.startswith(ns) for ns in self._reserved_namespaces):
+            msg = _("Attribute '%s' is reserved.") % path_root
+            raise webob.exc.HTTPForbidden(explanation=msg)
 
         if change['op'] == 'remove':
             return
@@ -1063,7 +1313,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         if len(path) != limits.get(op, 1):
             msg = _("Invalid JSON pointer for this resource: "
                     "'/%s'") % '/'.join(path)
-            raise webob.exc.HTTPBadRequest(explanation=six.text_type(msg))
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
     def _parse_json_schema_change(self, raw_change, draft_version):
         if draft_version == 10:
@@ -1299,6 +1549,10 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
 
 class ResponseSerializer(wsgi.JSONResponseSerializer):
+    # These properties will be filtered out from the response and not
+    # exposed to the client
+    _hidden_properties = ['os_glance_stage_host']
+
     def __init__(self, schema=None):
         super(ResponseSerializer, self).__init__()
         self.schema = schema or get_schema()
@@ -1318,7 +1572,8 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
                 return []
 
         try:
-            image_view = dict(image.extra_properties)
+            image_view = {k: v for k, v in dict(image.extra_properties).items()
+                          if k not in self._hidden_properties}
             attributes = ['name', 'disk_format', 'container_format',
                           'visibility', 'size', 'virtual_size', 'status',
                           'checksum', 'protected', 'min_ram', 'min_disk',
@@ -1399,14 +1654,12 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
 
     def show(self, response, image):
         image_view = self._format_image(image)
-        body = json.dumps(image_view, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = json.dumps(image_view, ensure_ascii=False)
         response.content_type = 'application/json'
 
     def update(self, response, image):
         image_view = self._format_image(image)
-        body = json.dumps(image_view, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = json.dumps(image_view, ensure_ascii=False)
         response.content_type = 'application/json'
 
     def index(self, response, result):
@@ -1424,8 +1677,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             params['marker'] = result['next_marker']
             next_query = urlparse.urlencode(params)
             body['next'] = '/v2/images?%s' % next_query
-        response.unicode_body = six.text_type(json.dumps(body,
-                                                         ensure_ascii=False))
+        response.unicode_body = json.dumps(body, ensure_ascii=False)
         response.content_type = 'application/json'
 
     def delete_from_store(self, response, result):
@@ -1669,7 +1921,7 @@ def load_custom_properties():
     else:
         msg = (_LW('Could not find schema properties file %s. Continuing '
                    'without custom properties') % filename)
-        LOG.warn(msg)
+        LOG.warning(msg)
         return {}
 
 
