@@ -13,13 +13,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import http.client as http
+import urllib.parse as urlparse
+
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
-import six
-from six.moves import http_client as http
-import six.moves.urllib.parse as urlparse
 import webob.exc
 from wsme.rest import json
 
@@ -98,13 +98,7 @@ class NamespaceController(object):
                 # Get resource type associations
                 filters = dict()
                 filters['namespace'] = db_namespace.namespace
-                try:
-                    repo_rs_type_list = rs_repo.list(filters=filters)
-                except exception.NotFound:
-                    # NOTE(danms): If we fail to list resource_types
-                    # for this namespace, do not fail the entire
-                    # namespace list operation with NotFound.
-                    repo_rs_type_list = []
+                repo_rs_type_list = rs_repo.list(filters=filters)
                 resource_type_list = [
                     ResourceTypeAssociation.to_wsme_model(
                         resource_type
@@ -692,7 +686,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
 
     def __render(self, json_data, response, response_status=None):
         body = jsonutils.dumps(json_data, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
         if response_status:
             response.status_int = response_status
