@@ -13,11 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import http.client as http
+
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from oslo_utils import encodeutils
-import six
-from six.moves import http_client as http
 import webob.exc
 from wsme.rest import json
 
@@ -26,6 +25,7 @@ from glance.api.v2.model.metadef_resource_type import ResourceType
 from glance.api.v2.model.metadef_resource_type import ResourceTypeAssociation
 from glance.api.v2.model.metadef_resource_type import ResourceTypeAssociations
 from glance.api.v2.model.metadef_resource_type import ResourceTypes
+from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import wsgi
 import glance.db
@@ -50,7 +50,17 @@ class ResourceTypeController(object):
         try:
             filters = {'namespace': None}
             rs_type_repo = self.gateway.get_metadef_resource_type_repo(
-                req.context)
+                req.context, authorization_layer=False)
+            # NOTE(abhishekk): Here we are just checking if user is
+            # authorized to view/list metadef resource types or not.
+            # Also there is no relation between list_metadef_resource_types
+            # and get_metadef_resource_type policies so can not enforce
+            # get_metadef_resource_type policy on individual resource
+            # type here.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                enforcer=self.policy).list_metadef_resource_types()
+
             db_resource_type_list = rs_type_repo.list(filters=filters)
             resource_type_list = [ResourceType.to_wsme_model(
                 resource_type) for resource_type in db_resource_type_list]
@@ -62,37 +72,76 @@ class ResourceTypeController(object):
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError(e)
         return resource_types
 
     def show(self, req, namespace):
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Here we are just checking if user is
+            # authorized to view/list metadef resource types or not.
+            # Each resource_type is checked against
+            # get_metadef_resource_type below.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).list_metadef_resource_types()
+
             filters = {'namespace': namespace}
             rs_type_repo = self.gateway.get_metadef_resource_type_repo(
-                req.context)
-            db_resource_type_list = rs_type_repo.list(filters=filters)
-            resource_type_list = [ResourceTypeAssociation.to_wsme_model(
-                resource_type) for resource_type in db_resource_type_list]
+                req.context, authorization_layer=False)
+            db_type_list = rs_type_repo.list(filters=filters)
+
+            rs_type_list = [
+                ResourceTypeAssociation.to_wsme_model(
+                    rs_type
+                ) for rs_type in db_type_list if api_policy.MetadefAPIPolicy(
+                    req.context, md_resource=rs_type.namespace,
+                    enforcer=self.policy
+                ).check('get_metadef_resource_type')]
+
             resource_types = ResourceTypeAssociations()
-            resource_types.resource_type_associations = resource_type_list
+            resource_types.resource_type_associations = rs_type_list
         except exception.Forbidden as e:
             LOG.debug("User not permitted to retrieve metadata resource types "
                       "within '%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError(e)
         return resource_types
 
     def create(self, req, resource_type, namespace):
         rs_type_factory = self.gateway.get_metadef_resource_type_factory(
-            req.context)
-        rs_type_repo = self.gateway.get_metadef_resource_type_repo(req.context)
+            req.context, authorization_layer=False)
+        rs_type_repo = self.gateway.get_metadef_resource_type_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Metadef resource type is created for Metadef
+            # namespaces. Here we are just checking if user is authorized
+            # to create metadef resource types or not.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).add_metadef_resource_type_association()
+
             new_resource_type = rs_type_factory.new_resource_type(
                 namespace=namespace, **resource_type.to_dict())
             rs_type_repo.add(new_resource_type)
@@ -105,14 +154,31 @@ class ResourceTypeController(object):
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
         return ResourceTypeAssociation.to_wsme_model(new_resource_type)
 
     def delete(self, req, namespace, resource_type):
-        rs_type_repo = self.gateway.get_metadef_resource_type_repo(req.context)
+        rs_type_repo = self.gateway.get_metadef_resource_type_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Metadef resource type is created for Metadef
+            # namespaces. Here we are just checking if user is authorized
+            # to delete metadef resource types or not.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy
+            ).remove_metadef_resource_type_association()
+
             filters = {}
             found = False
             filters['namespace'] = namespace
@@ -128,14 +194,11 @@ class ResourceTypeController(object):
             LOG.debug("User not permitted to delete metadata resource type "
                       "'%s' within '%s' namespace", resource_type, namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
-        except exception.NotFound as e:
+        except exception.NotFound:
             msg = (_("Failed to find resource type %(resourcetype)s to "
                      "delete") % {'resourcetype': resource_type})
             LOG.error(msg)
             raise webob.exc.HTTPNotFound(explanation=msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
@@ -178,20 +241,20 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def show(self, response, result):
         resource_type_json = json.tojson(ResourceTypeAssociations, result)
         body = jsonutils.dumps(resource_type_json, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
 
     def index(self, response, result):
         resource_type_json = json.tojson(ResourceTypes, result)
         body = jsonutils.dumps(resource_type_json, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
 
     def create(self, response, result):
         resource_type_json = json.tojson(ResourceTypeAssociation, result)
         response.status_int = http.CREATED
         body = jsonutils.dumps(resource_type_json, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
 
     def delete(self, response, result):
