@@ -13,17 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import http.client as http
+
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
-import six
-from six.moves import http_client as http
+from oslo_utils import strutils
 import webob.exc
 from wsme.rest import json
 
 from glance.api import policy
 from glance.api.v2.model.metadef_tag import MetadefTag
 from glance.api.v2.model.metadef_tag import MetadefTags
+from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import wsgi
 from glance.common import wsme_utils
@@ -48,14 +50,34 @@ class TagsController(object):
         self.tag_schema_link = '/v2/schemas/metadefs/tag'
 
     def create(self, req, namespace, tag_name):
-        tag_factory = self.gateway.get_metadef_tag_factory(req.context)
-        tag_repo = self.gateway.get_metadef_tag_repo(req.context)
+        tag_factory = self.gateway.get_metadef_tag_factory(
+            req.context, authorization_layer=False)
+        tag_repo = self.gateway.get_metadef_tag_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
+        try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
         tag_name_as_dict = {'name': tag_name}
         try:
             self.schema.validate(tag_name_as_dict)
         except exception.InvalidObject as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         try:
+            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
+            # Here we are just checking if user is authorized to create metadef
+            # tag or not.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).add_metadef_tag()
+
             new_meta_tag = tag_factory.new_tag(
                 namespace=namespace,
                 **tag_name_as_dict)
@@ -72,21 +94,41 @@ class TagsController(object):
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
         return MetadefTag.to_wsme_model(new_meta_tag)
 
     def create_tags(self, req, metadata_tags, namespace):
-        tag_factory = self.gateway.get_metadef_tag_factory(req.context)
-        tag_repo = self.gateway.get_metadef_tag_repo(req.context)
+        tag_factory = self.gateway.get_metadef_tag_factory(
+            req.context, authorization_layer=False)
+        tag_repo = self.gateway.get_metadef_tag_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
+            # Here we are just checking if user is authorized to create metadef
+            # tag or not.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).add_metadef_tags()
+
+            can_append = strutils.bool_from_string(req.headers.get(
+                'X-Openstack-Append'))
+
             tag_list = []
             for metadata_tag in metadata_tags.tags:
                 tag_list.append(tag_factory.new_tag(
                     namespace=namespace, **metadata_tag.to_dict()))
-            tag_repo.add_tags(tag_list)
+            tag_repo.add_tags(tag_list, can_append)
             tag_list_out = [MetadefTag(**{'name': db_metatag.name})
                             for db_metatag in tag_list]
             metadef_tags = MetadefTags()
@@ -99,19 +141,35 @@ class TagsController(object):
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
         return metadef_tags
 
     def index(self, req, namespace, marker=None, limit=None,
               sort_key='created_at', sort_dir='desc', filters=None):
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): This is just a "do you have permission to
+            # list tags" check. Each object is checked against
+            # get_metadef_tag below.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).get_metadef_tags()
+
             filters = filters or dict()
             filters['namespace'] = namespace
 
-            tag_repo = self.gateway.get_metadef_tag_repo(req.context)
+            tag_repo = self.gateway.get_metadef_tag_repo(
+                req.context, authorization_layer=False)
             if marker:
                 metadef_tag = tag_repo.get(namespace, marker)
                 marker = metadef_tag.tag_id
@@ -131,15 +189,31 @@ class TagsController(object):
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
         return metadef_tags
 
     def show(self, req, namespace, tag_name):
-        meta_tag_repo = self.gateway.get_metadef_tag_repo(req.context)
+        meta_tag_repo = self.gateway.get_metadef_tag_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Metadef tags are associated with
+            # namespace, so made provision to pass namespace here
+            # for visibility check
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).get_metadef_tag()
+
             metadef_tag = meta_tag_repo.get(namespace, tag_name)
             return MetadefTag.to_wsme_model(metadef_tag)
         except exception.Forbidden as e:
@@ -148,13 +222,29 @@ class TagsController(object):
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
     def update(self, req, metadata_tag, namespace, tag_name):
-        meta_repo = self.gateway.get_metadef_tag_repo(req.context)
+        meta_repo = self.gateway.get_metadef_tag_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
+            # Here we are just checking if user is authorized to update metadef
+            # tag or not.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).modify_metadef_tag()
+
             metadef_tag = meta_repo.get(namespace, tag_name)
             metadef_tag._old_name = metadef_tag.name
             metadef_tag.name = wsme_utils._get_value(
@@ -172,15 +262,31 @@ class TagsController(object):
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
         return MetadefTag.to_wsme_model(updated_metadata_tag)
 
     def delete(self, req, namespace, tag_name):
-        meta_repo = self.gateway.get_metadef_tag_repo(req.context)
+        meta_repo = self.gateway.get_metadef_tag_repo(
+            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(
+            req.context, authorization_layer=False)
         try:
+            namespace_obj = ns_repo.get(namespace)
+        except (exception.Forbidden, exception.NotFound):
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # namespace is outside of this user's project
+            msg = _("Namespace %s not found") % namespace
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
+            # Here we are just checking if user is authorized to delete metadef
+            # tag or not.
+            api_policy.MetadefAPIPolicy(
+                req.context,
+                md_resource=namespace_obj,
+                enforcer=self.policy).delete_metadef_tag()
+
             metadef_tag = meta_repo.get(namespace, tag_name)
             metadef_tag.delete()
             meta_repo.remove(metadef_tag)
@@ -190,9 +296,6 @@ class TagsController(object):
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
-        except Exception as e:
-            LOG.error(encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPInternalServerError()
 
 
 def _get_base_definitions():
@@ -203,7 +306,7 @@ def _get_base_properties():
     return {
         "name": {
             "type": "string",
-            "maxLength": 255
+            "maxLength": 80
         },
         "created_at": {
             "type": "string",
@@ -373,13 +476,13 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         response.status_int = http.CREATED
         metadata_tags_json = json.tojson(MetadefTags, result)
         body = jsonutils.dumps(metadata_tags_json, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
 
     def show(self, response, metadata_tag):
         metadata_tag_json = json.tojson(MetadefTag, metadata_tag)
         body = jsonutils.dumps(metadata_tag_json, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
 
     def update(self, response, metadata_tag):
@@ -389,7 +492,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def index(self, response, result):
         metadata_tags_json = json.tojson(MetadefTags, result)
         body = jsonutils.dumps(metadata_tags_json, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = body
         response.content_type = 'application/json'
 
     def delete(self, response, result):
