@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from glance.api.v2 import cached_images
 from glance.api.v2 import discovery
 from glance.api.v2 import image_actions
 from glance.api.v2 import image_data
@@ -416,6 +417,11 @@ class API(wsgi.Router):
                        action='show',
                        conditions={'method': ['GET']},
                        body_reject=True)
+        mapper.connect('/images/{image_id}/tasks',
+                       controller=images_resource,
+                       action='get_task_info',
+                       conditions={'method': ['GET']},
+                       body_reject=True)
         mapper.connect('/images/{image_id}',
                        controller=images_resource,
                        action='delete',
@@ -583,5 +589,46 @@ class API(wsgi.Router):
                        controller=reject_method_resource,
                        action='reject',
                        allowed_methods='GET')
+        mapper.connect('/info/usage',
+                       controller=info_resource,
+                       action='get_usage',
+                       conditions={'method': ['GET']})
+        mapper.connect('/info/stores/detail',
+                       controller=info_resource,
+                       action='get_stores_detail',
+                       conditions={'method': ['GET']},
+                       body_reject=True)
+        mapper.connect('/info/stores/detail',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='GET')
+
+        # Cache Management API
+        cache_manage_resource = cached_images.create_resource()
+        mapper.connect('/cache',
+                       controller=cache_manage_resource,
+                       action='get_cache_state',
+                       conditions={'method': ['GET']},
+                       body_reject=True)
+        mapper.connect('/cache',
+                       controller=cache_manage_resource,
+                       action='clear_cache',
+                       conditions={'method': ['DELETE']})
+        mapper.connect('/cache',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='GET, DELETE')
+        mapper.connect('/cache/{image_id}',
+                       controller=cache_manage_resource,
+                       action='delete_cache_entry',
+                       conditions={'method': ['DELETE']})
+        mapper.connect('/cache/{image_id}',
+                       controller=cache_manage_resource,
+                       action='queue_image_from_api',
+                       conditions={'method': ['PUT']})
+        mapper.connect('/cache/{image_id}',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='DELETE, PUT')
 
         super(API, self).__init__(mapper)
