@@ -15,6 +15,8 @@
 # under the License.
 
 import copy
+import http.client as http
+import urllib.parse as urlparse
 
 import debtcollector
 import glance_store
@@ -23,9 +25,6 @@ from oslo_log import log as logging
 import oslo_serialization.jsonutils as json
 from oslo_utils import encodeutils
 from oslo_utils import uuidutils
-import six
-from six.moves import http_client as http
-import six.moves.urllib.parse as urlparse
 import webob.exc
 
 from glance.api import common
@@ -92,7 +91,7 @@ class TasksController(object):
         except exception.Forbidden as e:
             msg = (_LW("Forbidden to create task. Reason: %(reason)s")
                    % {'reason': encodeutils.exception_to_unicode(e)})
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         return new_task
 
@@ -119,10 +118,10 @@ class TasksController(object):
                 result['next_marker'] = tasks[-1].task_id
         except (exception.NotFound, exception.InvalidSortKey,
                 exception.InvalidFilterRangeValue) as e:
-            LOG.warn(encodeutils.exception_to_unicode(e))
+            LOG.warning(encodeutils.exception_to_unicode(e))
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.Forbidden as e:
-            LOG.warn(encodeutils.exception_to_unicode(e))
+            LOG.warning(encodeutils.exception_to_unicode(e))
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         result['tasks'] = tasks
         return result
@@ -138,14 +137,14 @@ class TasksController(object):
             msg = (_LW("Failed to find task %(task_id)s. Reason: %(reason)s")
                    % {'task_id': task_id,
                       'reason': encodeutils.exception_to_unicode(e)})
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
             msg = (_LW("Forbidden to get task %(task_id)s. Reason:"
                        " %(reason)s")
                    % {'task_id': task_id,
                       'reason': encodeutils.exception_to_unicode(e)})
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         return task
 
@@ -270,8 +269,6 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
 
     def _inject_location_header(self, response, task):
         location = self._get_task_location(task)
-        if six.PY2:
-            location = location.encode('utf-8')
         response.headers['Location'] = location
 
     def _get_task_location(self, task):
@@ -325,8 +322,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
 
     def get(self, response, task):
         task_view = self._format_task(self.task_schema, task)
-        body = json.dumps(task_view, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
+        response.unicode_body = json.dumps(task_view, ensure_ascii=False)
         response.content_type = 'application/json'
 
     def index(self, response, result):
@@ -345,8 +341,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             params['marker'] = result['next_marker']
             next_query = urlparse.urlencode(params)
             body['next'] = '/v2/tasks?%s' % next_query
-        response.unicode_body = six.text_type(json.dumps(body,
-                                                         ensure_ascii=False))
+        response.unicode_body = json.dumps(body, ensure_ascii=False)
         response.content_type = 'application/json'
 
 
