@@ -13,11 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import http.client
+import urllib
+
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from six.moves import http_client
-from six.moves import urllib
 import webob.dec
 
 from glance.common import wsgi
@@ -76,29 +77,40 @@ class Controller(object):
             }
 
         version_objs = []
-        if CONF.enable_v2_api:
-            if CONF.enabled_backends:
-                version_objs.extend([
-                    build_version_object(2.10, 'v2', 'CURRENT'),
-                    build_version_object(2.9, 'v2', 'SUPPORTED'),
-                    build_version_object(2.8, 'v2', 'SUPPORTED')
-                ])
-            else:
-                version_objs.extend([
-                    build_version_object(2.9, 'v2', 'CURRENT'),
-                ])
+        if CONF.image_cache_dir:
             version_objs.extend([
-                build_version_object(2.7, 'v2', 'SUPPORTED'),
-                build_version_object(2.6, 'v2', 'SUPPORTED'),
-                build_version_object(2.5, 'v2', 'SUPPORTED'),
-                build_version_object(2.4, 'v2', 'SUPPORTED'),
-                build_version_object(2.3, 'v2', 'SUPPORTED'),
-                build_version_object(2.2, 'v2', 'SUPPORTED'),
-                build_version_object(2.1, 'v2', 'SUPPORTED'),
-                build_version_object(2.0, 'v2', 'SUPPORTED'),
+                build_version_object(2.15, 'v2', 'CURRENT'),
+                build_version_object(2.14, 'v2', 'SUPPORTED'),
             ])
+        else:
+            version_objs.extend([
+                build_version_object(2.15, 'v2', 'CURRENT'),
+            ])
+        if CONF.enabled_backends:
+            version_objs.extend([
+                build_version_object(2.13, 'v2', 'SUPPORTED'),
+                build_version_object(2.12, 'v2', 'SUPPORTED'),
+                build_version_object(2.11, 'v2', 'SUPPORTED'),
+                build_version_object('2.10', 'v2', 'SUPPORTED'),
+                build_version_object(2.9, 'v2', 'SUPPORTED'),
+                build_version_object(2.8, 'v2', 'SUPPORTED'),
+            ])
+        else:
+            version_objs.extend([
+                build_version_object(2.9, 'v2', 'SUPPORTED'),
+            ])
+        version_objs.extend([
+            build_version_object(2.7, 'v2', 'SUPPORTED'),
+            build_version_object(2.6, 'v2', 'SUPPORTED'),
+            build_version_object(2.5, 'v2', 'SUPPORTED'),
+            build_version_object(2.4, 'v2', 'SUPPORTED'),
+            build_version_object(2.3, 'v2', 'SUPPORTED'),
+            build_version_object(2.2, 'v2', 'SUPPORTED'),
+            build_version_object(2.1, 'v2', 'SUPPORTED'),
+            build_version_object(2.0, 'v2', 'SUPPORTED'),
+        ])
 
-        status = explicit and http_client.OK or http_client.MULTIPLE_CHOICES
+        status = explicit and http.client.OK or http.client.MULTIPLE_CHOICES
         response = webob.Response(request=req,
                                   status=status,
                                   content_type='application/json')
