@@ -25,7 +25,6 @@ from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import timeutils
 from oslo_utils import units
-import six
 import taskflow
 from taskflow.patterns import linear_flow as lf
 from taskflow import retry
@@ -553,7 +552,7 @@ class _ImportToStore(task.Task):
         """
         # NOTE(flaper87): Let's dance... and fall
         #
-        # Unfortunatelly, because of the way our domain layers work and
+        # Unfortunately, because of the way our domain layers work and
         # the checks done in the FS store, we can't simply rename the file
         # and set the location. To do that, we'd have to duplicate the logic
         # of every and each of the domain factories (quota, location, etc)
@@ -607,7 +606,7 @@ class _ImportToStore(task.Task):
                                   set_active=self.set_active,
                                   callback=self._status_callback)
         # NOTE(yebinama): set_image_data catches Exception and raises from
-        # them. Can't be more specific on exceptions catched.
+        # them. Can't be more specific on exceptions caught.
         except Exception:
             if self.all_stores_must_succeed:
                 raise
@@ -704,12 +703,12 @@ class _CompleteTask(task.Task):
             # necessary
             log_msg = _LE("Task ID %(task_id)s failed. Error: %(exc_type)s: "
                           "%(e)s")
-            LOG.exception(log_msg, {'exc_type': six.text_type(type(e)),
+            LOG.exception(log_msg, {'exc_type': str(type(e)),
                                     'e': encodeutils.exception_to_unicode(e),
                                     'task_id': task.task_id})
 
             err_msg = _("Error: %(exc_type)s: %(e)s")
-            task.fail(err_msg % {'exc_type': six.text_type(type(e)),
+            task.fail(err_msg % {'exc_type': str(type(e)),
                                  'e': encodeutils.exception_to_unicode(e)})
         finally:
             self.task_repo.save(task)
