@@ -23,7 +23,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-import six
 from stevedore import named
 from taskflow.patterns import linear_flow as lf
 from taskflow import retry
@@ -95,15 +94,22 @@ class _ImportToFS(task.Task):
         super(_ImportToFS, self).__init__(
             name='%s-ImportToFS-%s' % (task_type, task_id))
 
-        if CONF.task.work_dir is None:
-            msg = (_("%(task_id)s of %(task_type)s not configured "
-                     "properly. Missing work dir: %(work_dir)s") %
-                   {'task_id': self.task_id,
-                    'task_type': self.task_type,
-                    'work_dir': CONF.task.work_dir})
-            raise exception.BadTaskConfiguration(msg)
+        # NOTE(abhishekk): Use reserved 'os_glance_tasks_store' for tasks,
+        # the else part will be removed once old way of configuring store
+        # is deprecated.
+        if CONF.enabled_backends:
+            self.store = store_api.get_store_from_store_identifier(
+                'os_glance_tasks_store')
+        else:
+            if CONF.task.work_dir is None:
+                msg = (_("%(task_id)s of %(task_type)s not configured "
+                         "properly. Missing work dir: %(work_dir)s") %
+                       {'task_id': self.task_id,
+                        'task_type': self.task_type,
+                        'work_dir': CONF.task.work_dir})
+                raise exception.BadTaskConfiguration(msg)
 
-        self.store = self._build_store()
+            self.store = self._build_store()
 
     def _build_store(self):
         # NOTE(flaper87): Due to the nice glance_store api (#sarcasm), we're
@@ -185,7 +191,10 @@ class _ImportToFS(task.Task):
             return
 
         if os.path.exists(result.split("file://")[-1]):
-            store_api.delete_from_backend(result)
+            if CONF.enabled_backends:
+                store_api.delete(result, 'os_glance_tasks_store')
+            else:
+                store_api.delete_from_backend(result)
 
 
 class _DeleteFromFS(task.Task):
@@ -201,16 +210,20 @@ class _DeleteFromFS(task.Task):
 
         :param file_path: path to the file being deleted
         """
-        store_api.delete_from_backend(file_path)
+        if CONF.enabled_backends:
+            store_api.delete(file_path, 'os_glance_tasks_store')
+        else:
+            store_api.delete_from_backend(file_path)
 
 
 class _ImportToStore(task.Task):
 
-    def __init__(self, task_id, task_type, image_repo, uri):
+    def __init__(self, task_id, task_type, image_repo, uri, backend):
         self.task_id = task_id
         self.task_type = task_type
         self.image_repo = image_repo
         self.uri = uri
+        self.backend = backend
         super(_ImportToStore, self).__init__(
             name='%s-ImportToStore-%s' % (task_type, task_id))
 
@@ -275,7 +288,7 @@ class _ImportToStore(task.Task):
 
         # NOTE(flaper87): Let's dance... and fall
         #
-        # Unfortunatelly, because of the way our domain layers work and
+        # Unfortunately, because of the way our domain layers work and
         # the checks done in the FS store, we can't simply rename the file
         # and set the location. To do that, we'd have to duplicate the logic
         # of every and each of the domain factories (quota, location, etc)
@@ -311,7 +324,8 @@ class _ImportToStore(task.Task):
         # image_import.set_image_data(image, image_path, None)
         try:
             image_import.set_image_data(image,
-                                        file_path or self.uri, self.task_id)
+                                        file_path or self.uri, self.task_id,
+                                        backend=self.backend)
         except IOError as e:
             msg = (_('Uploading the image failed due to: %(exc)s') %
                    {'exc': encodeutils.exception_to_unicode(e)})
@@ -373,12 +387,12 @@ class _CompleteTask(task.Task):
             # necessary
             log_msg = _LE("Task ID %(task_id)s failed. Error: %(exc_type)s: "
                           "%(e)s")
-            LOG.exception(log_msg, {'exc_type': six.text_type(type(e)),
+            LOG.exception(log_msg, {'exc_type': str(type(e)),
                                     'e': encodeutils.exception_to_unicode(e),
                                     'task_id': task.task_id})
 
             err_msg = _("Error: %(exc_type)s: %(e)s")
-            task.fail(err_msg % {'exc_type': six.text_type(type(e)),
+            task.fail(err_msg % {'exc_type': str(type(e)),
                                  'e': encodeutils.exception_to_unicode(e)})
         finally:
             self.task_repo.save(task)
@@ -423,11 +437,13 @@ def get_flow(**kwargs):
     image_repo = kwargs.get('image_repo')
     image_factory = kwargs.get('image_factory')
     uri = kwargs.get('uri')
+    backend = kwargs.get('backend')
 
     flow = lf.Flow(task_type, retry=retry.AlwaysRevert()).add(
         _CreateImage(task_id, task_type, task_repo, image_repo, image_factory))
 
-    import_to_store = _ImportToStore(task_id, task_type, image_repo, uri)
+    import_to_store = _ImportToStore(task_id, task_type, image_repo, uri,
+                                     backend)
 
     try:
         # NOTE(flaper87): ImportToLocal and DeleteFromLocal shouldn't be here.
