@@ -13,12 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import futurist
+import urllib
+
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-from six.moves import urllib
 from stevedore import driver
 from taskflow import engines
 from taskflow.listeners import logging as llistener
@@ -87,13 +87,16 @@ CONF.register_opts(taskflow_executor_opts, group='taskflow_executor')
 
 class TaskExecutor(glance.async_.TaskExecutor):
 
-    def __init__(self, context, task_repo, image_repo, image_factory):
+    def __init__(self, context, task_repo, image_repo, image_factory,
+                 admin_repo=None):
         self.context = context
         self.task_repo = task_repo
         self.image_repo = image_repo
         self.image_factory = image_factory
+        self.admin_repo = admin_repo
         super(TaskExecutor, self).__init__(context, task_repo, image_repo,
-                                           image_factory)
+                                           image_factory,
+                                           admin_repo=admin_repo)
 
     @staticmethod
     def _fetch_an_executor():
@@ -101,13 +104,8 @@ class TaskExecutor(glance.async_.TaskExecutor):
             return None
         else:
             max_workers = CONF.taskflow_executor.max_workers
-            try:
-                return futurist.GreenThreadPoolExecutor(
-                    max_workers=max_workers)
-            except RuntimeError:
-                # NOTE(harlowja): I guess eventlet isn't being made
-                # useable, well just use native threads then (or try to).
-                return futurist.ThreadPoolExecutor(max_workers=max_workers)
+            threadpool_cls = glance.async_.get_threadpool_model()
+            return threadpool_cls(max_workers).pool
 
     def _get_flow(self, task):
         try:
@@ -123,6 +121,9 @@ class TaskExecutor(glance.async_.TaskExecutor):
                 'backend': task_input.get('backend')
             }
 
+            if self.admin_repo:
+                kwds['admin_repo'] = self.admin_repo
+
             if task.type == "import":
                 uri = script_utils.validate_location_uri(
                     task_input.get('import_from'))
@@ -137,8 +138,13 @@ class TaskExecutor(glance.async_.TaskExecutor):
             raise exception.ImportTaskError(message=exc.reason)
         except (exception.BadStoreUri, exception.Invalid) as exc:
             raise exception.ImportTaskError(message=exc.msg)
+        except exception.LimitExceeded as exc:
+            raise exception.ImportTaskError(message=exc.msg)
         except RuntimeError:
             raise NotImplementedError()
+        except Exception as e:
+            LOG.exception(_LE('Task initialization failed: %s'), str(e))
+            raise
 
     def begin_processing(self, task_id):
         try:
