@@ -39,6 +39,13 @@ if os.name == 'nt':
 else:
     eventlet.patcher.monkey_patch()
 
+# Monkey patch the original current_thread to use the up-to-date _active
+# global variable. See https://bugs.launchpad.net/bugs/1863021 and
+# https://github.com/eventlet/eventlet/issues/592
+import __original_module_threading as orig_threading
+import threading
+orig_threading.current_thread.__globals__['_active'] = threading._active
+
 from oslo_reports import guru_meditation_report as gmr
 from oslo_utils import encodeutils
 
@@ -55,6 +62,7 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import osprofiler.initializer
 
+import glance.async_
 from glance.common import config
 from glance.common import exception
 from glance.common import wsgi
@@ -100,6 +108,17 @@ def main():
                 host=CONF.bind_host
             )
 
+        if CONF.enforce_secure_rbac != CONF.oslo_policy.enforce_new_defaults:
+            fail_message = (
+                "[DEFAULT] enforce_secure_rbac does not match "
+                "[oslo_policy] enforce_new_defaults. Please set both to "
+                "True to enable secure RBAC personas. Otherwise, make sure "
+                "both are False.")
+            raise exception.ServerError(fail_message)
+
+        # NOTE(danms): Configure system-wide threading model to use eventlet
+        glance.async_.set_threadpool_model('eventlet')
+
         # NOTE(abhishekk): Added initialize_prefetcher KW argument to Server
         # object so that prefetcher object should only be initialized in case
         # of API service and ignored in case of registry. Once registry is
