@@ -19,7 +19,6 @@ Routines for configuring Glance
 
 import logging
 import os
-import sys
 
 from oslo_config import cfg
 from oslo_middleware import cors
@@ -37,7 +36,7 @@ paste_deploy_opts = [
 Deployment flavor to use in the server application pipeline.
 
 Provide a string value representing the appropriate deployment
-flavor used in the server application pipleline. This is typically
+flavor used in the server application pipeline. This is typically
 the partial name of a pipeline in the paste configuration file with
 the service name removed.
 
@@ -58,7 +57,7 @@ Related Options:
 Name of the paste configuration file.
 
 Provide a string value representing the name of the paste
-configuration file to use for configuring piplelines for
+configuration file to use for configuring pipelines for
 server application deployments.
 
 NOTES:
@@ -350,7 +349,7 @@ Related options:
     * location_strategy
 
 """)),
-    # NOTE(flaper87): The policy.yaml file should be updated and the locaiton
+    # NOTE(flaper87): The policy.yaml file should be updated and the location
     # related rules set to admin only once this option is finally removed.
     # NOTE(rosmaita): Unfortunately, this option is used to gate some code
     # paths; if the location related policies are set admin-only, then no
@@ -641,13 +640,13 @@ may overwhelm other system resources such as disk or outbound network
 bandwidth. If this is too small, image import requests will have to wait
 until a thread becomes available to begin processing.""")),
     cfg.StrOpt('python_interpreter',
-               default=sys.executable,
+               default=None,
                help=_("""
 Path to the python interpreter to use when spawning external
-processes. By default this is sys.executable, which should be the
-same interpreter running Glance itself. However, in some situations
-(i.e. uwsgi) this may not actually point to a python interpreter
-itself.""")),
+processes. If left unspecified, this will be sys.executable, which should
+be the same interpreter running Glance itself. However, in some situations
+(for example, uwsgi) sys.executable may not actually point to a python
+interpreter and an alternative value must be set.""")),
 ]
 
 
