@@ -16,8 +16,7 @@
 
 """Glance exception subclasses"""
 
-import six
-import six.moves.urllib.parse as urlparse
+import urllib.parse as urlparse
 
 from glance.i18n import _
 
@@ -54,12 +53,6 @@ class GlanceException(Exception):
         self.msg = message
         super(GlanceException, self).__init__(message)
 
-    def __unicode__(self):
-        # NOTE(flwang): By default, self.msg is an instance of Message, which
-        # can't be converted by str(). Based on the definition of
-        # __unicode__, it should return unicode always.
-        return six.text_type(self.msg)
-
 
 class MissingCredentialError(GlanceException):
     message = _("Missing required credential: %(required)s")
@@ -359,6 +352,10 @@ class ImportTaskError(TaskException, Invalid):
     message = _("An import task exception occurred")
 
 
+class TaskAbortedError(ImportTaskError):
+    message = _("Task was aborted externally")
+
+
 class DuplicateLocation(Duplicate):
     message = _("The location %(location)s already exists")
 
