@@ -21,7 +21,6 @@ from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-import six
 
 from glance.api.v2 import images as v2_api
 from glance.common import exception
@@ -65,7 +64,7 @@ def _execute(t_id, task_repo, image_repo, image_factory):
 
         # TODO(nikhil): need to bring back save_and_reraise_exception when
         # necessary
-        err_msg = ("Error: " + six.text_type(type(e)) + ': ' +
+        err_msg = ("Error: " + str(type(e)) + ': ' +
                    encodeutils.exception_to_unicode(e))
         log_msg = _LE(err_msg + ("Task ID %s" % task.task_id))  # noqa
         LOG.exception(log_msg)
@@ -152,9 +151,10 @@ def set_image_data(image, uri, task_id, backend=None, set_active=True,
         image.set_data(data_iter, backend=backend, set_active=set_active)
     except Exception as e:
         with excutils.save_and_reraise_exception():
-            LOG.warn(_LW("Task %(task_id)s failed with exception %(error)s") %
-                     {"error": encodeutils.exception_to_unicode(e),
-                      "task_id": task_id})
+            LOG.warning(_LW("Task %(task_id)s failed with exception "
+                            "%(error)s") %
+                        {"error": encodeutils.exception_to_unicode(e),
+                         "task_id": task_id})
             LOG.info(_LI("Task %(task_id)s: Could not import image file"
                          " %(image_data)s"), {"image_data": uri,
                                               "task_id": task_id})
