@@ -34,22 +34,8 @@ CONF.import_opt('image_size_cap', 'glance.common.config')
 CONF.import_opt('metadata_encryption_key', 'glance.common.config')
 
 
-def get_api(v1_mode=False):
-    """
-    When using v2_registry with v2_api or alone, it is essential that the opt
-    'data_api' be set to 'glance.db.registry.api'. This requires us to
-    differentiate what this method returns as the db api. i.e., we do not want
-    to return 'glance.db.registry.api' for a call from v1 api.
-    Reference bug #1516706
-    """
-    if v1_mode:
-        # prevent v1_api from talking to v2_registry.
-        if CONF.data_api == 'glance.db.simple.api':
-            api = importutils.import_module(CONF.data_api)
-        else:
-            api = importutils.import_module('glance.db.sqlalchemy.api')
-    else:
-        api = importutils.import_module(CONF.data_api)
+def get_api():
+    api = importutils.import_module('glance.db.sqlalchemy.api')
 
     if hasattr(api, 'configure'):
         api.configure()
@@ -119,6 +105,18 @@ class ImageRepo(object):
             key = CONF.metadata_encryption_key
             for l in locations:
                 l['url'] = crypt.urlsafe_decrypt(key, l['url'])
+
+        # NOTE(danms): If the image is shared and we are not the
+        # owner, we must have found it because we are a member. Set
+        # our tenant on the image as 'member' for policy checks in the
+        # upper layers. For any other image stage, we found the image
+        # some other way, so leave member=None.
+        if (db_image['visibility'] == 'shared' and
+                self.context.owner != db_image['owner']):
+            member = self.context.owner
+        else:
+            member = None
+
         return glance.domain.Image(
             image_id=db_image['id'],
             name=db_image['name'],
@@ -141,6 +139,7 @@ class ImageRepo(object):
             extra_properties=properties,
             tags=db_tags,
             os_hidden=db_image['os_hidden'],
+            member=member,
         )
 
     def _format_image_to_db(self, image):
@@ -196,17 +195,13 @@ class ImageRepo(object):
         if (image_values['size'] is not None
            and image_values['size'] > CONF.image_size_cap):
             raise exception.ImageSizeLimitExceeded
-        try:
-            new_values = self.db_api.image_update(self.context,
-                                                  image.image_id,
-                                                  image_values,
-                                                  purge_props=True,
-                                                  from_state=from_state,
-                                                  atomic_props=(
-                                                      IMAGE_ATOMIC_PROPS))
-        except (exception.ImageNotFound, exception.Forbidden):
-            msg = _("No image found with ID %s") % image.image_id
-            raise exception.ImageNotFound(msg)
+        new_values = self.db_api.image_update(self.context,
+                                              image.image_id,
+                                              image_values,
+                                              purge_props=True,
+                                              from_state=from_state,
+                                              atomic_props=(
+                                                  IMAGE_ATOMIC_PROPS))
         self.db_api.image_tag_set_all(self.context, image.image_id,
                                       image.tags)
         image.updated_at = new_values['updated_at']
@@ -357,6 +352,9 @@ class TaskRepo(object):
             task_input=db_task['input'],
             result=db_task['result'],
             message=db_task['message'],
+            image_id=db_task['image_id'],
+            user_id=db_task['user_id'],
+            request_id=db_task['request_id'],
         )
 
     def _format_task_stub_from_db(self, db_task):
@@ -368,6 +366,9 @@ class TaskRepo(object):
             expires_at=db_task['expires_at'],
             created_at=db_task['created_at'],
             updated_at=db_task['updated_at'],
+            image_id=db_task['image_id'],
+            user_id=db_task['user_id'],
+            request_id=db_task['request_id'],
         )
 
     def _format_task_to_db(self, task):
@@ -381,6 +382,9 @@ class TaskRepo(object):
                 'expires_at': task.expires_at,
                 'created_at': task.created_at,
                 'updated_at': task.updated_at,
+                'image_id': task.image_id,
+                'request_id': task.request_id,
+                'user_id': task.user_id,
                 }
         return task
 
@@ -469,12 +473,8 @@ class MetadefNamespaceRepo(object):
         )
 
     def get(self, namespace):
-        try:
-            db_api_namespace = self.db_api.metadef_namespace_get(
-                self.context, namespace)
-        except (exception.NotFound, exception.Forbidden):
-            msg = _('Could not find namespace %s') % namespace
-            raise exception.NotFound(msg)
+        db_api_namespace = self.db_api.metadef_namespace_get(
+            self.context, namespace)
         return self._format_namespace_from_db(db_api_namespace)
 
     def list(self, marker=None, limit=None, sort_key='created_at',
@@ -846,7 +846,7 @@ class MetadefTagRepo(object):
             self._format_metadef_tag_to_db(metadata_tag)
         )
 
-    def add_tags(self, metadata_tags):
+    def add_tags(self, metadata_tags, can_append=False):
         tag_list = []
         namespace = None
         for metadata_tag in metadata_tags:
@@ -855,7 +855,7 @@ class MetadefTagRepo(object):
                 namespace = metadata_tag.namespace
 
         self.db_api.metadef_tag_create_tags(
-            self.context, namespace, tag_list)
+            self.context, namespace, tag_list, can_append)
 
     def get(self, namespace, name):
         try:
