@@ -20,7 +20,6 @@ import uuid
 
 from oslo_config import cfg
 from oslo_log import log as logging
-import six
 
 from glance.common import exception
 from glance.common import timeutils
@@ -312,7 +311,7 @@ def _filter_images(images, filters, context,
                 continue
 
         to_add = True
-        for k, value in six.iteritems(filters):
+        for k, value in filters.items():
             key = k
             if k.endswith('_min') or k.endswith('_max'):
                 key = key[0:-4]
@@ -433,7 +432,7 @@ def image_set_property_atomic(image_id, name, value):
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        LOG.warn(_LW('Could not find image %s'), image_id)
+        LOG.warning(_LW('Could not find image %s'), image_id)
         raise exception.ImageNotFound()
 
     prop = _image_property_format(image_id,
@@ -446,7 +445,7 @@ def image_delete_property_atomic(image_id, name, value):
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        LOG.warn(_LW('Could not find image %s'), image_id)
+        LOG.warning(_LW('Could not find image %s'), image_id)
         raise exception.ImageNotFound()
 
     for i, prop in enumerate(image['properties']):
@@ -461,16 +460,16 @@ def _image_get(context, image_id, force_show_deleted=False, status=None):
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        LOG.warn(_LW('Could not find image %s'), image_id)
+        LOG.warning(_LW('Could not find image %s'), image_id)
         raise exception.ImageNotFound()
 
     if image['deleted'] and not (force_show_deleted
                                  or context.can_see_deleted):
-        LOG.warn(_LW('Unable to get deleted image'))
+        LOG.warning(_LW('Unable to get deleted image'))
         raise exception.ImageNotFound()
 
     if not is_image_visible(context, image):
-        LOG.warn(_LW('Unable to get unowned image'))
+        LOG.warning(_LW('Unable to get unowned image'))
         raise exception.Forbidden("Image not visible to you")
 
     return image
@@ -677,7 +676,7 @@ def image_location_update(context, image_id, location):
     if not updated:
         msg = (_("No location found with ID %(loc)s from image %(img)s") %
                dict(loc=loc_id, img=image_id))
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.NotFound(msg)
 
 
@@ -703,7 +702,7 @@ def image_location_delete(context, image_id, location_id, status,
     if not deleted:
         msg = (_("No location found with ID %(loc)s from image %(img)s") %
                dict(loc=location_id, img=image_id))
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.NotFound(msg)
 
 
@@ -991,12 +990,12 @@ def _task_get(context, task_id, force_show_deleted=False):
         task = DATA['tasks'][task_id]
     except KeyError:
         msg = _LW('Could not find task %s') % task_id
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
     if task['deleted'] and not (force_show_deleted or context.can_see_deleted):
         msg = _LW('Unable to get deleted task %s') % task_id
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
     if not _is_task_visible(context, task):
@@ -1092,7 +1091,7 @@ def _filter_tasks(tasks, filters, context, admin_as_user=False):
             continue
 
         add = True
-        for k, value in six.iteritems(filters):
+        for k, value in filters.items():
             add = task[k] == value and task['deleted'] is False
             if not add:
                 break
@@ -1172,7 +1171,7 @@ def _task_info_get(task_id):
         task_info = DATA['task_info'][task_id]
     except KeyError:
         msg = _LW('Could not find task info %s') % task_id
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
     return task_info
@@ -1258,7 +1257,7 @@ def metadef_namespace_get_by_id(context, namespace_id):
     except StopIteration:
         msg = (_("Metadata definition namespace not found for id=%s")
                % namespace_id)
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.MetadefNamespaceNotFound(msg)
 
     if not _is_namespace_visible(context, namespace):
@@ -1389,7 +1388,7 @@ def metadef_object_get_by_id(context, namespace_name, object_id):
     else:
         msg = (_("Metadata definition object not found for id=%s")
                % object_id)
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.MetadefObjectNotFound(msg)
 
 
@@ -1645,7 +1644,7 @@ def metadef_property_get_by_id(context, namespace_name, property_id):
     else:
         msg = (_("Metadata definition property not found for id=%s")
                % property_id)
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.MetadefPropertyNotFound(msg)
 
 
@@ -1850,7 +1849,7 @@ def metadef_tag_get_by_id(context, namespace_name, id):
             return tag
     else:
         msg = (_("Metadata definition tag not found for id=%s") % id)
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exception.MetadefTagNotFound(msg)
 
 
@@ -1911,7 +1910,8 @@ def metadef_tag_create(context, namespace_name, values):
 
 
 @log_call
-def metadef_tag_create_tags(context, namespace_name, tag_list):
+def metadef_tag_create_tags(context, namespace_name, tag_list,
+                            can_append=False):
     """Create a metadef tag"""
     global DATA
 
@@ -1922,6 +1922,12 @@ def metadef_tag_create_tags(context, namespace_name, tag_list):
     allowed_attributes = ['name']
     data_tag_list = []
     tag_name_list = []
+    if can_append:
+        # NOTE(mrjoshi): We need to fetch existing tags here for duplicate
+        # check while adding new one
+        tag_name_list = [tag['name']
+                         for tag in metadef_tag_get_all(context,
+                                                        namespace_name)]
     for tag_value in tag_list:
         tag_values = copy.deepcopy(tag_value)
         tag_name = tag_values['name']
@@ -1946,8 +1952,8 @@ def metadef_tag_create_tags(context, namespace_name, tag_list):
 
         tag_values['namespace_id'] = namespace['id']
         data_tag_list.append(_format_tag(tag_values))
-
-    DATA['metadef_tags'] = []
+    if not can_append:
+        DATA['metadef_tags'] = []
     for tag in data_tag_list:
         DATA['metadef_tags'].append(tag)
 
