@@ -12,20 +12,19 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
-from __future__ import with_statement
 from logging import config as log_config
 
 from alembic import context
-from sqlalchemy import engine_from_config, pool
+from oslo_config import cfg
+from oslo_db.sqlalchemy import enginefacade
 
 from glance.db.sqlalchemy import models
-from glance.db.sqlalchemy import models_glare
 from glance.db.sqlalchemy import models_metadef
 
 # this is the Alembic Config object, which provides
 # access to the values within the .ini file in use.
 config = context.config
+CONF = cfg.CONF
 
 # other values from the config, defined by the needs of env.py,
 # can be acquired:
@@ -39,8 +38,6 @@ log_config.fileConfig(config.config_file_name)
 # add your model's MetaData object here
 # for 'autogenerate' support
 target_metadata = models.BASE.metadata
-for table in models_glare.BASE.metadata.sorted_tables:
-    target_metadata._add_table(table.name, table.schema, table)
 for table in models_metadef.BASE_DICT.metadata.sorted_tables:
     target_metadata._add_table(table.name, table.schema, table)
 
@@ -57,7 +54,7 @@ def run_migrations_offline():
     script output.
 
     """
-    url = config.get_main_option("sqlalchemy.url")
+    url = CONF.database.connection
     context.configure(
         url=url, target_metadata=target_metadata, literal_binds=True)
 
@@ -72,12 +69,9 @@ def run_migrations_online():
     and associate a connection with the context.
 
     """
-    connectable = engine_from_config(
-        config.get_section(config.config_ini_section),
-        prefix='sqlalchemy.',
-        poolclass=pool.NullPool)
+    engine = enginefacade.writer.get_engine()
 
-    with connectable.connect() as connection:
+    with engine.connect() as connection:
         context.configure(
             connection=connection,
             target_metadata=target_metadata
@@ -86,6 +80,7 @@ def run_migrations_online():
         with context.begin_transaction():
             context.run_migrations()
 
+
 if context.is_offline_mode():
     run_migrations_offline()
 else:
