@@ -87,7 +87,7 @@ def _get(context, namespace_id, session):
     except sa_orm.exc.NoResultFound:
         msg = (_("Metadata definition namespace not found for id=%s")
                % namespace_id)
-        LOG.warn(msg)
+        LOG.warning(msg)
         raise exc.MetadefNamespaceNotFound(msg)
 
     # Make sure they are allowed to view it.
@@ -182,7 +182,7 @@ def _get_all_by_resource_types(context, session, filters, marker=None,
     for name, namespace_id in db_recs:
         namespace_id_list.append(namespace_id)
 
-    if len(namespace_id_list) is 0:
+    if len(namespace_id_list) == 0:
         return []
 
     filters2 = filters
