@@ -17,13 +17,12 @@
 LRU Cache for Image Data
 """
 
-import hashlib
-
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 
 from glance.common import exception
@@ -173,11 +172,11 @@ class ImageCache(object):
             self.driver_class = importutils.import_class(driver_module)
             LOG.info(_LI("Image cache loaded driver '%s'."), driver_name)
         except ImportError as import_err:
-            LOG.warn(_LW("Image cache driver "
-                         "'%(driver_name)s' failed to load. "
-                         "Got error: '%(import_err)s."),
-                     {'driver_name': driver_name,
-                      'import_err': import_err})
+            LOG.warning(_LW("Image cache driver "
+                            "'%(driver_name)s' failed to load. "
+                            "Got error: '%(import_err)s."),
+                        {'driver_name': driver_name,
+                         'import_err': import_err})
 
             driver_module = __name__ + '.drivers.sqlite.Driver'
             LOG.info(_LI("Defaulting to SQLite driver."))
@@ -194,11 +193,11 @@ class ImageCache(object):
             self.driver.configure()
         except exception.BadDriverConfiguration as config_err:
             driver_module = self.driver_class.__module__
-            LOG.warn(_LW("Image cache driver "
-                         "'%(driver_module)s' failed to configure. "
-                         "Got error: '%(config_err)s"),
-                     {'driver_module': driver_module,
-                      'config_err': config_err})
+            LOG.warning(_LW("Image cache driver "
+                            "'%(driver_module)s' failed to configure. "
+                            "Got error: '%(config_err)s"),
+                        {'driver_module': driver_module,
+                         'config_err': config_err})
             LOG.info(_LI("Defaulting to SQLite driver."))
             default_module = __name__ + '.drivers.sqlite.Driver'
             self.driver_class = importutils.import_class(default_module)
@@ -347,7 +346,7 @@ class ImageCache(object):
 
     def cache_tee_iter(self, image_id, image_iter, image_checksum):
         try:
-            current_checksum = hashlib.md5()
+            current_checksum = md5(usedforsecurity=False)
 
             with self.driver.open_for_write(image_id) as cache_file:
                 for chunk in image_iter:
