@@ -51,7 +51,8 @@ class Prefetcher(base.CacheApp):
             return False
 
         if image.status != 'active':
-            LOG.warn(_LW("Image '%s' is not active. Not caching.") % image_id)
+            LOG.warning(_LW("Image '%s' is not active. Not caching.") %
+                        image_id)
             return False
 
         for loc in image.locations:
@@ -85,8 +86,8 @@ class Prefetcher(base.CacheApp):
         results = pool.map(self.fetch_image_into_cache, images)
         successes = sum([1 for r in results if r is True])
         if successes != num_images:
-            LOG.warn(_LW("Failed to successfully cache all "
-                         "images in queue."))
+            LOG.warning(_LW("Failed to successfully cache all "
+                            "images in queue."))
             return False
 
         LOG.info(_LI("Successfully cached all %d images"), num_images)
