@@ -22,7 +22,6 @@ from oslo_log import log as logging
 import oslo_messaging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-import six
 import webob
 
 from glance.common import exception
@@ -61,14 +60,16 @@ prefix to disable all event notifications within a group.
 Possible values:
     A comma-separated list of individual notification types or
     notification groups to be disabled. Currently supported groups:
-        * image
-        * image.member
-        * task
-        * metadef_namespace
-        * metadef_object
-        * metadef_property
-        * metadef_resource_type
-        * metadef_tag
+
+    * image
+    * image.member
+    * task
+    * metadef_namespace
+    * metadef_object
+    * metadef_property
+    * metadef_resource_type
+    * metadef_tag
+
     For a complete listing and description of each event refer to:
     http://docs.openstack.org/developer/glance/notifications.html
 
@@ -217,7 +218,7 @@ def format_metadef_namespace_notification(metadef_namespace):
 def format_metadef_object_notification(metadef_object):
     object_properties = metadef_object.properties or {}
     properties = []
-    for name, prop in six.iteritems(object_properties):
+    for name, prop in object_properties.items():
         object_property = _format_metadef_object_property(name, prop)
         properties.append(object_property)
 
@@ -328,8 +329,7 @@ class NotificationBase(object):
         _send_notification(self.notifier.info, notification_id, payload)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class NotificationProxy(NotificationBase):
+class NotificationProxy(NotificationBase, metaclass=abc.ABCMeta):
     def __init__(self, repo, context, notifier):
         self.repo = repo
         self.context = context
@@ -343,8 +343,7 @@ class NotificationProxy(NotificationBase):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class NotificationRepoProxy(NotificationBase):
+class NotificationRepoProxy(NotificationBase, metaclass=abc.ABCMeta):
     def __init__(self, repo, context, notifier):
         self.repo = repo
         self.context = context
@@ -364,8 +363,7 @@ class NotificationRepoProxy(NotificationBase):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class NotificationFactoryProxy(object):
+class NotificationFactoryProxy(metaclass=abc.ABCMeta):
     def __init__(self, factory, context, notifier):
         kwargs = {'context': context, 'notifier': notifier}
 
@@ -398,6 +396,17 @@ class ImageProxy(NotificationProxy, domain_proxy.Image):
             'receiver_user_id': self.context.user_id,
         }
 
+    def _format_import_properties(self):
+        importing = self.repo.extra_properties.get(
+            'os_glance_importing_to_stores')
+        importing = importing.split(',') if importing else []
+        failed = self.repo.extra_properties.get('os_glance_failed_import')
+        failed = failed.split(',') if failed else []
+        return {
+            'os_glance_importing_to_stores': importing,
+            'os_glance_failed_import': failed
+        }
+
     def _get_chunk_data_iterator(self, data, chunk_size=None):
         sent = 0
         for chunk in data:
@@ -424,12 +433,15 @@ class ImageProxy(NotificationProxy, domain_proxy.Image):
         data = self.repo.get_data(offset=offset, chunk_size=chunk_size)
         return self._get_chunk_data_iterator(data, chunk_size=chunk_size)
 
-    def set_data(self, data, size=None, backend=None):
-        self.send_notification('image.prepare', self.repo, backend=backend)
+    def set_data(self, data, size=None, backend=None, set_active=True):
+        self.send_notification('image.prepare', self.repo, backend=backend,
+                               extra_payload=self._format_import_properties())
 
         notify_error = self.notifier.error
+        status = self.repo.status
         try:
-            self.repo.set_data(data, size, backend=backend)
+            self.repo.set_data(data, size, backend=backend,
+                               set_active=set_active)
         except glance_store.StorageFull as e:
             msg = (_("Image storage media is full: %s") %
                    encodeutils.exception_to_unicode(e))
@@ -484,8 +496,11 @@ class ImageProxy(NotificationProxy, domain_proxy.Image):
                         'error': encodeutils.exception_to_unicode(e)})
                 _send_notification(notify_error, 'image.upload', msg)
         else:
-            self.send_notification('image.upload', self.repo)
-            self.send_notification('image.activate', self.repo)
+            extra_payload = self._format_import_properties()
+            self.send_notification('image.upload', self.repo,
+                                   extra_payload=extra_payload)
+            if set_active and status != 'active':
+                self.send_notification('image.activate', self.repo)
 
 
 class ImageMemberProxy(NotificationProxy, domain_proxy.ImageMember):
@@ -898,8 +913,9 @@ class MetadefTagRepoProxy(NotificationRepoProxy, domain_proxy.MetadefTagRepo):
         self.send_notification('metadef_tag.create', metadef_tag)
         return result
 
-    def add_tags(self, metadef_tags):
-        result = super(MetadefTagRepoProxy, self).add_tags(metadef_tags)
+    def add_tags(self, metadef_tags, can_append=False):
+        result = super(MetadefTagRepoProxy, self).add_tags(metadef_tags,
+                                                           can_append)
         for metadef_tag in metadef_tags:
             self.send_notification('metadef_tag.create', metadef_tag)
 
