@@ -29,7 +29,6 @@ import os
 import platform
 import shutil
 import signal
-import six
 import socket
 import subprocess
 import sys
@@ -38,6 +37,7 @@ from testtools import content as ttc
 import textwrap
 import time
 from unittest import mock
+import urllib.parse as urlparse
 import uuid
 
 import fixtures
@@ -45,9 +45,6 @@ import glance_store
 from os_win import utilsfactory as os_win_utilsfactory
 from oslo_config import cfg
 from oslo_serialization import jsonutils
-# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
-from six.moves import range
-import six.moves.urllib.parse as urlparse
 import testtools
 import webob
 
@@ -78,8 +75,7 @@ except RuntimeError:
     pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class BaseServer(object):
+class BaseServer(metaclass=abc.ABCMeta):
     """
     Class used to easily manage starting and stopping
     a server during functional test runs.
@@ -797,7 +793,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         self.api_protocol = 'http'
         self.api_port, api_sock = test_utils.get_unused_port_and_socket()
         # NOTE: Scrubber is enabled by default for the functional tests.
-        # Please disbale it by explicitly setting 'self.include_scrubber' to
+        # Please disable it by explicitly setting 'self.include_scrubber' to
         # False in the test SetUps that do not require Scrubber to run.
         self.include_scrubber = True
 
@@ -1152,7 +1148,7 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         self.api_protocol = 'http'
         self.api_port, api_sock = test_utils.get_unused_port_and_socket()
         # NOTE: Scrubber is enabled by default for the functional tests.
-        # Please disbale it by explicitly setting 'self.include_scrubber' to
+        # Please disable it by explicitly setting 'self.include_scrubber' to
         # False in the test SetUps that do not require Scrubber to run.
         self.include_scrubber = True
 
@@ -1554,6 +1550,8 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             CacheManageFilter.factory
             [pipeline:glance-api-cachemanagement]
             pipeline = context cache cachemanage rootapp
+            [pipeline:glance-api-caching]
+            pipeline = context cache rootapp
             [pipeline:glance-api]
             pipeline = context rootapp
             [composite:rootapp]
