@@ -13,18 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import uuid
+import http.client as http
 
 from oslo_serialization import jsonutils
 import requests
-from six.moves import http_client as http
 
-from glance.tests import functional
-
-TENANT1 = str(uuid.uuid4())
+from glance.tests.functional.v2 import metadef_base
 
 
-class TestMetadefObjects(functional.FunctionalTest):
+class TestMetadefObjects(metadef_base.MetadefFunctionalTestBase):
 
     def setUp(self):
         super(TestMetadefObjects, self).setUp()
@@ -32,20 +29,6 @@ class TestMetadefObjects(functional.FunctionalTest):
         self.api_server.deployment_flavor = 'noauth'
         self.start_servers(**self.__dict__.copy())
 
-    def _url(self, path):
-        return 'http://127.0.0.1:%d%s' % (self.api_port, path)
-
-    def _headers(self, custom_headers=None):
-        base_headers = {
-            'X-Identity-Status': 'Confirmed',
-            'X-Auth-Token': '932c5c84-02ac-4fe5-a9ba-620af0e2bb96',
-            'X-User-Id': 'f9a41d13-0c13-47e9-bee2-ce4e8bfe958e',
-            'X-Tenant-Id': TENANT1,
-            'X-Roles': 'admin',
-        }
-        base_headers.update(custom_headers or {})
-        return base_headers
-
     def test_metadata_objects_lifecycle(self):
         # Namespace should not exist
         path = self._url('/v2/metadefs/namespaces/MyNamespace')
@@ -271,3 +254,176 @@ class TestMetadefObjects(functional.FunctionalTest):
                          (namespace_name, metadata_object_name))
         response = requests.get(path, headers=self._headers())
         self.assertEqual(http.NOT_FOUND, response.status_code)
+
+    def _create_object(self, namespaces):
+        objects = []
+        for namespace in namespaces:
+            headers = self._headers({'X-Tenant-Id': namespace['owner']})
+            data = {
+                "name": "object_of_%s" % (namespace['namespace']),
+                "description": "object description.",
+                "required": [
+                    "property1"
+                ],
+                "properties": {
+                    "property1": {
+                        "type": "integer",
+                        "title": "property1",
+                        "description": "property1 description",
+                    },
+                }
+            }
+            path = self._url('/v2/metadefs/namespaces/%s/objects' %
+                             namespace['namespace'])
+            response = requests.post(path, headers=headers, json=data)
+            self.assertEqual(http.CREATED, response.status_code)
+            obj_metadata = response.json()
+            metadef_objects = dict()
+            metadef_objects[namespace['namespace']] = obj_metadata['name']
+            objects.append(metadef_objects)
+
+        return objects
+
+    def _update_object(self, path, headers, data, namespace):
+        response = requests.put(path, headers=headers, json=data)
+        self.assertEqual(http.OK, response.status_code, response.text)
+
+        expected_object = {
+            'description': data['description'],
+            'name': data['name'],
+            'properties': data['properties'],
+            'required': data['required'],
+            'schema': '/v2/schemas/metadefs/object',
+            'self': '/v2/metadefs/namespaces/%s/objects/%s' % (namespace,
+                                                               data['name'])
+        }
+        # Returned metadata_object should reflect the changes
+        metadata_object = response.json()
+        metadata_object.pop('created_at')
+        metadata_object.pop('updated_at')
+        self.assertEqual(metadata_object, expected_object)
+
+        # Updates should persist across requests
+        response = requests.get(path, headers=self._headers())
+        metadata_object = response.json()
+        metadata_object.pop('created_at')
+        metadata_object.pop('updated_at')
+        self.assertEqual(metadata_object, expected_object)
+
+    def test_role_base_metadata_objects_lifecycle(self):
+        # Create public and private namespaces for tenant1 and tenant2
+        path = self._url('/v2/metadefs/namespaces')
+        headers = self._headers({'content-type': 'application/json'})
+        tenant1_namespaces = []
+        tenant2_namespaces = []
+        for tenant in [self.tenant1, self.tenant2]:
+            headers['X-Tenant-Id'] = tenant
+            for visibility in ['public', 'private']:
+                namespace_data = {
+                    "namespace": "%s_%s_namespace" % (tenant, visibility),
+                    "display_name": "My User Friendly Namespace",
+                    "description": "My description",
+                    "visibility": visibility,
+                    "owner": tenant
+                }
+                namespace = self.create_namespace(path, headers,
+                                                  namespace_data)
+                self.assertNamespacesEqual(namespace, namespace_data)
+                if tenant == self.tenant1:
+                    tenant1_namespaces.append(namespace)
+                else:
+                    tenant2_namespaces.append(namespace)
+
+        # Create a metadef object for each namespace created above
+        tenant1_objects = self._create_object(tenant1_namespaces)
+        tenant2_objects = self._create_object(tenant2_namespaces)
+
+        def _check_object_access(objects, tenant):
+            headers = self._headers({'content-type': 'application/json',
+                                     'X-Tenant-Id': tenant,
+                                     'X-Roles': 'reader,member'})
+            for obj in objects:
+                for namespace, object_name in obj.items():
+                    path = self._url('/v2/metadefs/namespaces/%s/objects/%s' %
+                                     (namespace, object_name))
+                    headers = headers
+                    response = requests.get(path, headers=headers)
+                    if namespace.split('_')[1] == 'public':
+                        expected = http.OK
+                    else:
+                        expected = http.NOT_FOUND
+
+                    self.assertEqual(expected, response.status_code)
+
+                    path = self._url(
+                        '/v2/metadefs/namespaces/%s/objects' % namespace)
+                    response = requests.get(path, headers=headers)
+                    self.assertEqual(expected, response.status_code)
+                    if expected == http.OK:
+                        resp_objs = response.json()['objects']
+                        self.assertEqual(
+                            sorted(obj.values()),
+                            sorted([x['name'] for x in resp_objs]))
+
+        # Check Tenant 1 can access objects of all public namespace
+        # and cannot access object of private namespace of Tenant 2
+        _check_object_access(tenant2_objects, self.tenant1)
+
+        # Check Tenant 2 can access objects of public namespace and
+        # cannot access objects of private namespace of Tenant 1
+        _check_object_access(tenant1_objects, self.tenant2)
+
+        # Update objects with admin and non admin role
+        total_objects = tenant1_objects + tenant2_objects
+        for obj in total_objects:
+            for namespace, object_name in obj.items():
+                data = {
+                    "name": object_name,
+                    "description": "desc-UPDATED",
+                    "required": [
+                        "property1"
+                    ],
+                    "properties": {
+                        'property1': {
+                            'type': 'integer',
+                            "title": "property1",
+                            'description': 'p1 desc-UPDATED',
+                        }
+                    }
+                }
+                # Update object should fail with non admin role
+                path = self._url('/v2/metadefs/namespaces/%s/objects/%s' %
+                                 (namespace, object_name))
+                headers['X-Roles'] = "reader,member"
+                response = requests.put(path, headers=headers, json=data)
+                self.assertEqual(http.FORBIDDEN, response.status_code)
+
+                # Should work with admin role
+                headers = self._headers({
+                    'X-Tenant-Id': namespace.split('_')[0]})
+                self._update_object(path, headers, data, namespace)
+
+        # Delete object should not be allowed to non admin role
+        for obj in total_objects:
+            for namespace, object_name in obj.items():
+                path = self._url('/v2/metadefs/namespaces/%s/objects/%s' %
+                                 (namespace, object_name))
+                response = requests.delete(
+                    path, headers=self._headers({
+                        'X-Roles': 'reader,member',
+                        'X-Tenant-Id': namespace.split('_')[0]
+                    }))
+                self.assertEqual(http.FORBIDDEN, response.status_code)
+
+        # Delete all metadef objects
+        headers = self._headers()
+        for obj in total_objects:
+            for namespace, object_name in obj.items():
+                path = self._url('/v2/metadefs/namespaces/%s/objects/%s' %
+                                 (namespace, object_name))
+                response = requests.delete(path, headers=headers)
+                self.assertEqual(http.NO_CONTENT, response.status_code)
+
+                # Deleted objects should not be exist
+                response = requests.get(path, headers=headers)
+                self.assertEqual(http.NOT_FOUND, response.status_code)
