@@ -13,9 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import http.client
+
 import eventlet
 from oslo_serialization import jsonutils as json
-from six.moves import http_client
 
 from glance.api.v2 import tasks
 from glance.common import timeutils
@@ -56,7 +57,6 @@ class TestTasksApi(base.ApiTest):
     def __init__(self, *args, **kwargs):
         super(TestTasksApi, self).__init__(*args, **kwargs)
         self.api_flavor = 'fakeauth'
-        self.registry_flavor = 'fakeauth'
 
     def _wait_on_task_execution(self, max_wait=5):
         """Wait until all the tasks have finished execution and are in
@@ -74,7 +74,7 @@ class TestTasksApi(base.ApiTest):
                                              headers=minimal_task_headers())
             content_dict = json.loads(content)
 
-            self.assertEqual(http_client.OK, res.status)
+            self.assertEqual(http.client.OK, res.status)
             res_tasks = content_dict['tasks']
             if len(res_tasks) != 0:
                 for task in res_tasks:
@@ -105,7 +105,7 @@ class TestTasksApi(base.ApiTest):
                                               headers=headers,
                                               body=body_content)
 
-        self.assertEqual(http_client.CREATED, response.status)
+        self.assertEqual(http.client.CREATED, response.status)
 
         task = json.loads(content)
         task_id = task['id']
@@ -127,7 +127,7 @@ class TestTasksApi(base.ApiTest):
                                               headers=minimal_task_headers())
         content_dict = json.loads(content)
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
         self.assertFalse(content_dict['tasks'])
 
         # 1. GET /tasks/{task_id}
@@ -137,7 +137,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.NOT_FOUND, response.status)
+        self.assertEqual(http.client.NOT_FOUND, response.status)
 
         # 2. POST /tasks
         # Create a new task
@@ -150,7 +150,7 @@ class TestTasksApi(base.ApiTest):
         path = "/v2/tasks/%s" % task_id
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         # NOTE(sabari): wait for all task executions to finish before checking
         # task status.
@@ -162,7 +162,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
         self.assertIsNotNone(content)
 
         data = json.loads(content)
@@ -185,7 +185,7 @@ class TestTasksApi(base.ApiTest):
         path = "/v2/schemas/task"
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         schema = tasks.get_task_schema()
         expected_schema = schema.minimal()
@@ -198,7 +198,7 @@ class TestTasksApi(base.ApiTest):
         path = "/v2/schemas/tasks"
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         schema = tasks.get_collection_schema()
         expected_schema = schema.minimal()
@@ -221,7 +221,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(
             path, 'POST', headers=minimal_task_headers(task_owner),
             body=body_content)
-        self.assertEqual(http_client.CREATED, response.status)
+        self.assertEqual(http.client.CREATED, response.status)
 
         data = json.loads(content)
         task_id = data['id']
@@ -242,7 +242,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(
             path, 'POST', headers=minimal_task_headers(task_owner),
             body=body_content)
-        self.assertEqual(http_client.BAD_REQUEST, response.status)
+        self.assertEqual(http.client.BAD_REQUEST, response.status)
 
         # 1. POST /tasks
         # Create a new task with invalid input for type 'import'
@@ -255,7 +255,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(
             path, 'POST', headers=minimal_task_headers(task_owner),
             body=body_content)
-        self.assertEqual(http_client.BAD_REQUEST, response.status)
+        self.assertEqual(http.client.BAD_REQUEST, response.status)
 
         # NOTE(nikhil): wait for all task executions to finish before exiting
         # else there is a risk of running into deadlock
@@ -269,7 +269,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         content_dict = json.loads(content)
         self.assertFalse(content_dict['tasks'])
@@ -291,7 +291,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         content_dict = json.loads(content)
         self.assertEqual(2, len(content_dict['tasks']))
@@ -304,7 +304,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         content_dict = json.loads(content)
         self.assertEqual(1, len(content_dict['tasks']))
@@ -317,7 +317,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         content_dict = json.loads(content)
         self.assertEqual(1, len(content_dict['tasks']))
@@ -330,7 +330,7 @@ class TestTasksApi(base.ApiTest):
 
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         content_dict = json.loads(content)
         self.assertEqual(2, len(content_dict['tasks']))
@@ -352,7 +352,7 @@ class TestTasksApi(base.ApiTest):
         path = "/v2/tasks"
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
         tasks = json.loads(content)
         self.assertFalse(tasks['tasks'])
 
@@ -375,7 +375,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         tasks = json.loads(content)['tasks']
 
@@ -388,7 +388,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         actual_tasks = json.loads(content)['tasks']
 
@@ -403,7 +403,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         actual_tasks = json.loads(content)['tasks']
 
@@ -418,7 +418,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         actual_tasks = json.loads(content)['tasks']
 
@@ -435,7 +435,7 @@ class TestTasksApi(base.ApiTest):
         path = "/v2/tasks"
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
         tasks = json.loads(content)
         self.assertFalse(tasks['tasks'])
 
@@ -459,7 +459,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         actual_tasks = json.loads(content)['tasks']
 
@@ -474,7 +474,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         expected_task_owners = [TENANT1, TENANT2, TENANT3]
         expected_task_owners.sort()
@@ -490,7 +490,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         actual_tasks = json.loads(content)['tasks']
         self.assertEqual(2, len(actual_tasks))
@@ -506,7 +506,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path, 'GET',
                                               headers=minimal_task_headers())
 
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
 
         actual_tasks = json.loads(content)['tasks']
 
@@ -527,7 +527,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(
             path, 'POST', headers=minimal_task_headers(task_owner),
             body=body_content)
-        self.assertEqual(http_client.CREATED, response.status)
+        self.assertEqual(http.client.CREATED, response.status)
 
         data = json.loads(content)
         task_id = data['id']
@@ -538,7 +538,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path,
                                               'DELETE',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.METHOD_NOT_ALLOWED, response.status)
+        self.assertEqual(http.client.METHOD_NOT_ALLOWED, response.status)
         self.assertEqual('GET', response.webob_resp.headers.get('Allow'))
         self.assertEqual(('GET',), response.webob_resp.allow)
         self.assertEqual(('GET',), response.allow)
@@ -549,7 +549,7 @@ class TestTasksApi(base.ApiTest):
         response, content = self.http.request(path,
                                               'GET',
                                               headers=minimal_task_headers())
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(http.client.OK, response.status)
         self.assertIsNotNone(content)
 
         # NOTE(nikhil): wait for all task executions to finish before exiting
