@@ -18,7 +18,6 @@ from unittest import mock
 
 from oslo_serialization import jsonutils
 import webob
-import wsme
 
 from glance.api import policy
 from glance.api.v2 import metadef_namespaces as namespaces
@@ -297,44 +296,6 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         expected = set([NAMESPACE1, NAMESPACE3])
         self.assertEqual(expected, actual)
 
-    def test_namespace_index_resource_type_delete_race(self):
-        request = unit_test_utils.get_fake_request()
-        filters = {'resource_types': [RESOURCE_TYPE1]}
-
-        real_gmrtr = self.namespace_controller.gateway.\
-            get_metadef_resource_type_repo
-
-        def race_delete(*a, **k):
-            self.db.metadef_namespace_delete(request.context, NAMESPACE3)
-            return real_gmrtr(*a, **k)
-
-        with mock.patch.object(self.namespace_controller.gateway,
-                               'get_metadef_resource_type_repo') as g:
-            # NOTE(danms): We simulate a late delete of one of our
-            # namespaces by hijacking the call to get the metadef RT
-            # repo and doing a delete at that point, before we iterate
-            # the list of namespaces we already pulled from the DB. If
-            # the code in the index API method changes, this will need
-            # to be updated.
-            g.side_effect = race_delete
-            output = self.namespace_controller.index(request, filters=filters)
-        output = output.to_dict()
-        self.assertEqual(2, len(output['namespaces']))
-        actual = set([namespace.namespace for namespace
-                      in output['namespaces']])
-        # We should still see both namespaces
-        expected = set([NAMESPACE1, NAMESPACE3])
-        self.assertEqual(expected, actual)
-
-        # And the first (undeleted) one should have the expected
-        # associations...
-        self.assertEqual(
-            1, len(output['namespaces'][0].resource_type_associations))
-        # ...but the one we deleted should be empty
-        self.assertEqual(
-            wsme.types.Unset,
-            output['namespaces'][1].resource_type_associations)
-
     def test_namespace_show(self):
         request = unit_test_utils.get_fake_request()
         output = self.namespace_controller.show(request, NAMESPACE1)
@@ -2041,6 +2002,94 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
             ]
         )
 
+    def test_tag_create_tags_with_append_true(self):
+        request = unit_test_utils.get_fake_request(
+            headers={'X-Openstack-Append': 'True'}, roles=['admin'])
+
+        metadef_tags = tags.MetadefTags()
+        # As TAG1 is already created in setup, just creating other two tags.
+        metadef_tags.tags = _db_tags_fixture([TAG2, TAG3])
+        output = self.tag_controller.create_tags(
+            request, metadef_tags, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(2, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG2, TAG3])
+        self.assertEqual(expected, actual)
+        self.assertNotificationLog(
+            "metadef_tag.create", [
+                {'name': TAG2, 'namespace': NAMESPACE1},
+                {'name': TAG3, 'namespace': NAMESPACE1},
+            ]
+        )
+
+        metadef_tags = tags.MetadefTags()
+        metadef_tags.tags = _db_tags_fixture([TAG4, TAG5])
+        output = self.tag_controller.create_tags(
+            request, metadef_tags, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(2, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG4, TAG5])
+        self.assertEqual(expected, actual)
+        self.assertNotificationLog(
+            "metadef_tag.create", [
+                {'name': TAG4, 'namespace': NAMESPACE1},
+                {'name': TAG5, 'namespace': NAMESPACE1},
+            ]
+        )
+
+        output = self.tag_controller.index(request, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(5, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG1, TAG2, TAG3, TAG4, TAG5])
+        self.assertEqual(expected, actual)
+
+    def test_tag_create_tags_with_append_false(self):
+        request = unit_test_utils.get_fake_request(
+            headers={'X-Openstack-Append': 'False'}, roles=['admin'])
+
+        metadef_tags = tags.MetadefTags()
+        # As TAG1 is already created in setup, just creating other two tags.
+        metadef_tags.tags = _db_tags_fixture([TAG2, TAG3])
+        output = self.tag_controller.create_tags(
+            request, metadef_tags, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(2, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG2, TAG3])
+        self.assertEqual(expected, actual)
+        self.assertNotificationLog(
+            "metadef_tag.create", [
+                {'name': TAG2, 'namespace': NAMESPACE1},
+                {'name': TAG3, 'namespace': NAMESPACE1},
+            ]
+        )
+
+        metadef_tags = tags.MetadefTags()
+        metadef_tags.tags = _db_tags_fixture([TAG4, TAG5])
+        output = self.tag_controller.create_tags(
+            request, metadef_tags, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(2, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG4, TAG5])
+        self.assertEqual(expected, actual)
+        self.assertNotificationLog(
+            "metadef_tag.create", [
+                {'name': TAG4, 'namespace': NAMESPACE1},
+                {'name': TAG5, 'namespace': NAMESPACE1},
+            ]
+        )
+
+        output = self.tag_controller.index(request, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(2, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG4, TAG5])
+        self.assertEqual(expected, actual)
+
     def test_tag_create_duplicate_tags(self):
         request = unit_test_utils.get_fake_request(roles=['admin'])
 
@@ -2087,6 +2136,42 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         expected = set([TAG1, TAG2, TAG3])
         self.assertEqual(expected, actual)
 
+    def test_tag_create_duplicate_with_pre_existing_tags_with_append(self):
+        request = unit_test_utils.get_fake_request(
+            headers={'X-Openstack-Append': 'True'}, roles=['admin'])
+
+        metadef_tags = tags.MetadefTags()
+        # As TAG1 is already created in setup, just creating other two tags.
+        metadef_tags.tags = _db_tags_fixture([TAG2, TAG3])
+        output = self.tag_controller.create_tags(
+            request, metadef_tags, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(2, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG2, TAG3])
+        self.assertEqual(expected, actual)
+        self.assertNotificationLog(
+            "metadef_tag.create", [
+                {'name': TAG2, 'namespace': NAMESPACE1},
+                {'name': TAG3, 'namespace': NAMESPACE1},
+            ]
+        )
+
+        metadef_tags = tags.MetadefTags()
+        metadef_tags.tags = _db_tags_fixture([TAG4, TAG5, TAG4])
+        self.assertRaises(
+            webob.exc.HTTPConflict,
+            self.tag_controller.create_tags,
+            request, metadef_tags, NAMESPACE1)
+        self.assertNotificationsLog([])
+
+        output = self.tag_controller.index(request, NAMESPACE1)
+        output = output.to_dict()
+        self.assertEqual(3, len(output['tags']))
+        actual = set([tag.name for tag in output['tags']])
+        expected = set([TAG1, TAG2, TAG3])
+        self.assertEqual(expected, actual)
+
     def test_tag_create_conflict(self):
         request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPConflict,
