@@ -17,9 +17,13 @@
 
 import errno
 import functools
+import http.client
+import http.server
+import io
 import os
 import shlex
 import shutil
+import signal
 import socket
 import subprocess
 import threading
@@ -32,10 +36,8 @@ from oslo_config import cfg
 from oslo_config import fixture as cfg_fixture
 from oslo_log.fixture import logging_error as log_fixture
 from oslo_log import log
+from oslo_utils import timeutils
 from oslo_utils import units
-import six
-from six.moves import BaseHTTPServer
-from six.moves import http_client as http
 import testtools
 import webob
 
@@ -50,6 +52,7 @@ from glance.db.sqlalchemy import api as db_api
 from glance.tests.unit import fixtures as glance_fixtures
 
 CONF = cfg.CONF
+LOG = log.getLogger(__name__)
 try:
     CONF.debug
 except cfg.NoSuchOptError:
@@ -92,7 +95,7 @@ class BaseTestCase(testtools.TestCase):
         self.useFixture(glance_fixtures.StandardLogging())
 
     def set_policy(self):
-        conf_file = "policy.json"
+        conf_file = "policy.yaml"
         self.policy_file = self._copy_data_file(conf_file, self.conf_dir)
         self.config(policy_file=self.policy_file, group='oslo_policy')
 
@@ -139,7 +142,7 @@ class BaseTestCase(testtools.TestCase):
         self._config_fixture.config(**kw)
 
     def mock_object(self, obj, attr_name, *args, **kwargs):
-        """"Use python mock to mock an object attirbute
+        """"Use python mock to mock an object attribute
 
         Mocks the specified objects attribute with the given value.
         Automatically performs 'addCleanup' for the mock.
@@ -266,10 +269,7 @@ def fork_exec(cmd,
                         os.dup2(fptr.fileno(), desc)
                     except OSError:
                         pass
-        if pass_fds and hasattr(os, 'set_inheritable'):
-            # os.set_inheritable() is only available and needed
-            # since Python 3.4. On Python 3.3 and older, file descriptors are
-            # inheritable by default.
+        if pass_fds:
             for fd in pass_fds:
                 os.set_inheritable(fd, True)
 
@@ -281,7 +281,8 @@ def fork_exec(cmd,
 
 def wait_for_fork(pid,
                   raise_error=True,
-                  expected_exitcode=0):
+                  expected_exitcode=0,
+                  force=True):
     """
     Wait for a process to complete
 
@@ -290,18 +291,52 @@ def wait_for_fork(pid,
     is raised.
     """
 
-    rc = 0
-    try:
-        (pid, rc) = os.waitpid(pid, 0)
-        rc = os.WEXITSTATUS(rc)
-        if rc != expected_exitcode:
-            raise RuntimeError('The exit code %d is not %d'
-                               % (rc, expected_exitcode))
-    except Exception:
-        if raise_error:
-            raise
+    # For the first period, we wait without being pushy, but after
+    # this timer expires, we start sending SIGTERM
+    term_timer = timeutils.StopWatch(5)
+    term_timer.start()
 
-    return rc
+    # After this timer expires we start sending SIGKILL
+    nice_timer = timeutils.StopWatch(7)
+    nice_timer.start()
+
+    # Process gets a maximum amount of time to exit before we fail the
+    # test
+    total_timer = timeutils.StopWatch(10)
+    total_timer.start()
+
+    while not total_timer.expired():
+        try:
+            cpid, rc = os.waitpid(pid, force and os.WNOHANG or 0)
+            if cpid == 0 and force:
+                if not term_timer.expired():
+                    # Waiting for exit on first signal
+                    pass
+                elif not nice_timer.expired():
+                    # Politely ask the process to GTFO
+                    LOG.warning('Killing child %i with SIGTERM' % pid)
+                    os.kill(pid, signal.SIGTERM)
+                else:
+                    # No more Mr. Nice Guy
+                    LOG.warning('Killing child %i with SIGKILL' % pid)
+                    os.kill(pid, signal.SIGKILL)
+                    expected_exitcode = signal.SIGKILL
+                time.sleep(1)
+                continue
+            LOG.info('waitpid(%i) returned %i,%i' % (pid, cpid, rc))
+            if rc != expected_exitcode:
+                raise RuntimeError('The exit code %d is not %d'
+                                   % (rc, expected_exitcode))
+            return rc
+        except ChildProcessError:
+            # Nothing to wait for
+            return 0
+        except Exception as e:
+            LOG.error('Got wait error: %s' % e)
+            if raise_error:
+                raise
+
+    raise RuntimeError('Gave up waiting for %i to exit!' % pid)
 
 
 def execute(cmd,
@@ -508,21 +543,21 @@ def minimal_add_command(port, name, suffix='', public=True):
 
 def start_http_server(image_id, image_data):
     def _get_http_handler_class(fixture):
-        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+        class StaticHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
             def do_GET(self):
-                self.send_response(http.OK)
+                self.send_response(http.client.OK)
                 self.send_header('Content-Length', str(len(fixture)))
                 self.end_headers()
-                self.wfile.write(six.b(fixture))
+                self.wfile.write(fixture.encode('latin-1'))
                 return
 
             def do_HEAD(self):
                 # reserve non_existing_image_path for the cases where we expect
                 # 404 from the server
                 if 'non_existing_image_path' in self.path:
-                    self.send_response(http.NOT_FOUND)
+                    self.send_response(http.client.NOT_FOUND)
                 else:
-                    self.send_response(http.OK)
+                    self.send_response(http.client.OK)
                 self.send_header('Content-Length', str(len(fixture)))
                 self.end_headers()
                 return
@@ -536,7 +571,7 @@ def start_http_server(image_id, image_data):
 
     server_address = ('127.0.0.1', 0)
     handler_class = _get_http_handler_class(image_data)
-    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    httpd = http.server.HTTPServer(server_address, handler_class)
     port = httpd.socket.getsockname()[1]
 
     thread = threading.Thread(target=httpd.serve_forever)
@@ -578,10 +613,10 @@ class FakeAuthMiddleware(wsgi.Middleware):
 
 
 class FakeHTTPResponse(object):
-    def __init__(self, status=http.OK, headers=None, data=None,
+    def __init__(self, status=http.client.OK, headers=None, data=None,
                  *args, **kwargs):
         data = data or b'I am a teapot, short and stout\n'
-        self.data = six.BytesIO(data)
+        self.data = io.BytesIO(data)
         self.read = self.data.read
         self.status = status
         self.headers = headers or {'content-length': len(data)}
@@ -670,10 +705,10 @@ def is_sqlite_version_prior_to(major, minor):
 
 def start_standalone_http_server():
     def _get_http_handler_class():
-        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+        class StaticHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
             def do_GET(self):
                 data = b"Hello World!!!"
-                self.send_response(http.OK)
+                self.send_response(http.client.OK)
                 self.send_header('Content-Length', str(len(data)))
                 self.end_headers()
                 self.wfile.write(data)
@@ -683,7 +718,7 @@ def start_standalone_http_server():
 
     server_address = ('127.0.0.1', 0)
     handler_class = _get_http_handler_class()
-    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    httpd = http.server.HTTPServer(server_address, handler_class)
     port = httpd.socket.getsockname()[1]
 
     thread = threading.Thread(target=httpd.serve_forever)
